import datetime
from typing import TYPE_CHECKING, Any, TypeVar, Union, cast
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.scorer_type import ScorerType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.alert import Alert
    from ..models.document import Document
    from ..models.feedback_aggregate import FeedbackAggregate
    from ..models.feedback_rating_db import FeedbackRatingDB
    from ..models.hallucination_segment import HallucinationSegment
    from ..models.job_info import JobInfo
    from ..models.metric_computing import MetricComputing
    from ..models.metric_error import MetricError
    from ..models.metric_failed import MetricFailed
    from ..models.metric_not_applicable import MetricNotApplicable
    from ..models.metric_not_computed import MetricNotComputed
    from ..models.metric_pending import MetricPending
    from ..models.metric_success import MetricSuccess
    from ..models.metric_threshold import MetricThreshold
    from ..models.segment import Segment


T = TypeVar("T", bound="PromptRunColumn")


@_attrs_define
class PromptRunColumn:
    """
    Attributes:
        name (str): Name of the column.
        alert (Union['Alert', None, Unset]): Alert for a given column with title and message
        data_type (Union[Unset, Any]): Data type of the column. This is used to determine how to format the data on the
            UI. Default: 'unknown'.
        description (Union[None, Unset, str]): Description of the column.
        display_values (Union[None, Unset, list[Union[None, str]]]): Display values of the column. This is used to
            display the values in the UI if we want them to be different from the actual values.
        filterable (Union[Unset, bool]): Whether the column is filterable by value. Doesn't include categorical filters
            Default: True.
        group_description (Union[None, Unset, str]): Description of the group (super column).
        group_label (Union[None, Unset, str]): Display label of the group (super column) in the UI.
        group_name (Union[None, Unset, str]): Name of the group (super column).
        job_info (Union[None, Unset, list[Union['JobInfo', None]]]): Job info used for computing the column. Only
            present for metrics columns that have a separate scorer job.
        job_type (Union[None, ScorerType, Unset]): If a column is associated with a basic or a plus scorer. Defaults to
            None for columns not associated with either.
        label (Union[None, Unset, str]): Display label of the column in the UI.
        metric_infos (Union[Unset, list[Union['MetricComputing', 'MetricError', 'MetricFailed', 'MetricNotApplicable',
            'MetricNotComputed', 'MetricPending', 'MetricSuccess']]]): Metric values with metadata.
        metric_threshold (Union['MetricThreshold', None, Unset]): Thresholds for the column, if this is a metrics
            column.
        sortable (Union[Unset, bool]): Whether the column is sortable. Default: True.
        values (Union[Unset, list[Union['Document', 'FeedbackAggregate', 'FeedbackRatingDB', 'HallucinationSegment',
            'Segment', None, UUID, bool, datetime.datetime, float, int, list[Union['Document', 'FeedbackAggregate',
            'FeedbackRatingDB', 'HallucinationSegment', 'Segment', None, UUID, bool, datetime.datetime, float, int, str]],
            list[list[Union['Document', 'FeedbackAggregate', 'FeedbackRatingDB', 'HallucinationSegment', 'Segment', None,
            UUID, bool, datetime.datetime, float, int, str]]], list[list[list[Union['Document', 'FeedbackAggregate',
            'FeedbackRatingDB', 'HallucinationSegment', 'Segment', None, UUID, bool, datetime.datetime, float, int, str]]]],
            str]]]): Values of the column.
    """

    name: str
    alert: Union["Alert", None, Unset] = UNSET
    data_type: Union[Unset, Any] = "unknown"
    description: Union[None, Unset, str] = UNSET
    display_values: Union[None, Unset, list[Union[None, str]]] = UNSET
    filterable: Union[Unset, bool] = True
    group_description: Union[None, Unset, str] = UNSET
    group_label: Union[None, Unset, str] = UNSET
    group_name: Union[None, Unset, str] = UNSET
    job_info: Union[None, Unset, list[Union["JobInfo", None]]] = UNSET
    job_type: Union[None, ScorerType, Unset] = UNSET
    label: Union[None, Unset, str] = UNSET
    metric_infos: Union[
        Unset,
        list[
            Union[
                "MetricComputing",
                "MetricError",
                "MetricFailed",
                "MetricNotApplicable",
                "MetricNotComputed",
                "MetricPending",
                "MetricSuccess",
            ]
        ],
    ] = UNSET
    metric_threshold: Union["MetricThreshold", None, Unset] = UNSET
    sortable: Union[Unset, bool] = True
    values: Union[
        Unset,
        list[
            Union[
                "Document",
                "FeedbackAggregate",
                "FeedbackRatingDB",
                "HallucinationSegment",
                "Segment",
                None,
                UUID,
                bool,
                datetime.datetime,
                float,
                int,
                list[
                    Union[
                        "Document",
                        "FeedbackAggregate",
                        "FeedbackRatingDB",
                        "HallucinationSegment",
                        "Segment",
                        None,
                        UUID,
                        bool,
                        datetime.datetime,
                        float,
                        int,
                        str,
                    ]
                ],
                list[
                    list[
                        Union[
                            "Document",
                            "FeedbackAggregate",
                            "FeedbackRatingDB",
                            "HallucinationSegment",
                            "Segment",
                            None,
                            UUID,
                            bool,
                            datetime.datetime,
                            float,
                            int,
                            str,
                        ]
                    ]
                ],
                list[
                    list[
                        list[
                            Union[
                                "Document",
                                "FeedbackAggregate",
                                "FeedbackRatingDB",
                                "HallucinationSegment",
                                "Segment",
                                None,
                                UUID,
                                bool,
                                datetime.datetime,
                                float,
                                int,
                                str,
                            ]
                        ]
                    ]
                ],
                str,
            ]
        ],
    ] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.alert import Alert
        from ..models.document import Document
        from ..models.feedback_aggregate import FeedbackAggregate
        from ..models.feedback_rating_db import FeedbackRatingDB
        from ..models.hallucination_segment import HallucinationSegment
        from ..models.job_info import JobInfo
        from ..models.metric_computing import MetricComputing
        from ..models.metric_error import MetricError
        from ..models.metric_not_applicable import MetricNotApplicable
        from ..models.metric_not_computed import MetricNotComputed
        from ..models.metric_pending import MetricPending
        from ..models.metric_success import MetricSuccess
        from ..models.metric_threshold import MetricThreshold
        from ..models.segment import Segment

        name = self.name

        alert: Union[None, Unset, dict[str, Any]]
        if isinstance(self.alert, Unset):
            alert = UNSET
        elif isinstance(self.alert, Alert):
            alert = self.alert.to_dict()
        else:
            alert = self.alert

        data_type = self.data_type

        description: Union[None, Unset, str]
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        display_values: Union[None, Unset, list[Union[None, str]]]
        if isinstance(self.display_values, Unset):
            display_values = UNSET
        elif isinstance(self.display_values, list):
            display_values = []
            for display_values_type_0_item_data in self.display_values:
                display_values_type_0_item: Union[None, str]
                display_values_type_0_item = display_values_type_0_item_data
                display_values.append(display_values_type_0_item)

        else:
            display_values = self.display_values

        filterable = self.filterable

        group_description: Union[None, Unset, str]
        if isinstance(self.group_description, Unset):
            group_description = UNSET
        else:
            group_description = self.group_description

        group_label: Union[None, Unset, str]
        if isinstance(self.group_label, Unset):
            group_label = UNSET
        else:
            group_label = self.group_label

        group_name: Union[None, Unset, str]
        if isinstance(self.group_name, Unset):
            group_name = UNSET
        else:
            group_name = self.group_name

        job_info: Union[None, Unset, list[Union[None, dict[str, Any]]]]
        if isinstance(self.job_info, Unset):
            job_info = UNSET
        elif isinstance(self.job_info, list):
            job_info = []
            for job_info_type_0_item_data in self.job_info:
                job_info_type_0_item: Union[None, dict[str, Any]]
                if isinstance(job_info_type_0_item_data, JobInfo):
                    job_info_type_0_item = job_info_type_0_item_data.to_dict()
                else:
                    job_info_type_0_item = job_info_type_0_item_data
                job_info.append(job_info_type_0_item)

        else:
            job_info = self.job_info

        job_type: Union[None, Unset, str]
        if isinstance(self.job_type, Unset):
            job_type = UNSET
        elif isinstance(self.job_type, ScorerType):
            job_type = self.job_type.value
        else:
            job_type = self.job_type

        label: Union[None, Unset, str]
        if isinstance(self.label, Unset):
            label = UNSET
        else:
            label = self.label

        metric_infos: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.metric_infos, Unset):
            metric_infos = []
            for metric_infos_item_data in self.metric_infos:
                metric_infos_item: dict[str, Any]
                if isinstance(metric_infos_item_data, MetricNotComputed):
                    metric_infos_item = metric_infos_item_data.to_dict()
                elif isinstance(metric_infos_item_data, MetricPending):
                    metric_infos_item = metric_infos_item_data.to_dict()
                elif isinstance(metric_infos_item_data, MetricComputing):
                    metric_infos_item = metric_infos_item_data.to_dict()
                elif isinstance(metric_infos_item_data, MetricNotApplicable):
                    metric_infos_item = metric_infos_item_data.to_dict()
                elif isinstance(metric_infos_item_data, MetricSuccess):
                    metric_infos_item = metric_infos_item_data.to_dict()
                elif isinstance(metric_infos_item_data, MetricError):
                    metric_infos_item = metric_infos_item_data.to_dict()
                else:
                    metric_infos_item = metric_infos_item_data.to_dict()

                metric_infos.append(metric_infos_item)

        metric_threshold: Union[None, Unset, dict[str, Any]]
        if isinstance(self.metric_threshold, Unset):
            metric_threshold = UNSET
        elif isinstance(self.metric_threshold, MetricThreshold):
            metric_threshold = self.metric_threshold.to_dict()
        else:
            metric_threshold = self.metric_threshold

        sortable = self.sortable

        values: Union[
            Unset,
            list[
                Union[
                    None,
                    bool,
                    dict[str, Any],
                    float,
                    int,
                    list[Union[None, bool, dict[str, Any], float, int, str]],
                    list[list[Union[None, bool, dict[str, Any], float, int, str]]],
                    list[list[list[Union[None, bool, dict[str, Any], float, int, str]]]],
                    str,
                ]
            ],
        ] = UNSET
        if not isinstance(self.values, Unset):
            values = []
            for values_item_data in self.values:
                values_item: Union[
                    None,
                    bool,
                    dict[str, Any],
                    float,
                    int,
                    list[Union[None, bool, dict[str, Any], float, int, str]],
                    list[list[Union[None, bool, dict[str, Any], float, int, str]]],
                    list[list[list[Union[None, bool, dict[str, Any], float, int, str]]]],
                    str,
                ]
                if isinstance(values_item_data, UUID):
                    values_item = str(values_item_data)
                elif isinstance(values_item_data, datetime.datetime):
                    values_item = values_item_data.isoformat()
                elif isinstance(values_item_data, Segment):
                    values_item = values_item_data.to_dict()
                elif isinstance(values_item_data, HallucinationSegment):
                    values_item = values_item_data.to_dict()
                elif isinstance(values_item_data, Document):
                    values_item = values_item_data.to_dict()
                elif isinstance(values_item_data, FeedbackRatingDB):
                    values_item = values_item_data.to_dict()
                elif isinstance(values_item_data, FeedbackAggregate):
                    values_item = values_item_data.to_dict()
                elif isinstance(values_item_data, list):
                    values_item = []
                    for values_item_type_11_item_data in values_item_data:
                        values_item_type_11_item: Union[None, bool, dict[str, Any], float, int, str]
                        if isinstance(values_item_type_11_item_data, UUID):
                            values_item_type_11_item = str(values_item_type_11_item_data)
                        elif isinstance(values_item_type_11_item_data, datetime.datetime):
                            values_item_type_11_item = values_item_type_11_item_data.isoformat()
                        elif isinstance(values_item_type_11_item_data, Segment):
                            values_item_type_11_item = values_item_type_11_item_data.to_dict()
                        elif isinstance(values_item_type_11_item_data, HallucinationSegment):
                            values_item_type_11_item = values_item_type_11_item_data.to_dict()
                        elif isinstance(values_item_type_11_item_data, Document):
                            values_item_type_11_item = values_item_type_11_item_data.to_dict()
                        elif isinstance(values_item_type_11_item_data, FeedbackRatingDB):
                            values_item_type_11_item = values_item_type_11_item_data.to_dict()
                        elif isinstance(values_item_type_11_item_data, FeedbackAggregate):
                            values_item_type_11_item = values_item_type_11_item_data.to_dict()
                        else:
                            values_item_type_11_item = values_item_type_11_item_data
                        values_item.append(values_item_type_11_item)

                elif isinstance(values_item_data, list):
                    values_item = []
                    for values_item_type_12_item_data in values_item_data:
                        values_item_type_12_item = []
                        for values_item_type_12_item_item_data in values_item_type_12_item_data:
                            values_item_type_12_item_item: Union[None, bool, dict[str, Any], float, int, str]
                            if isinstance(values_item_type_12_item_item_data, UUID):
                                values_item_type_12_item_item = str(values_item_type_12_item_item_data)
                            elif isinstance(values_item_type_12_item_item_data, datetime.datetime):
                                values_item_type_12_item_item = values_item_type_12_item_item_data.isoformat()
                            elif isinstance(values_item_type_12_item_item_data, Segment):
                                values_item_type_12_item_item = values_item_type_12_item_item_data.to_dict()
                            elif isinstance(values_item_type_12_item_item_data, HallucinationSegment):
                                values_item_type_12_item_item = values_item_type_12_item_item_data.to_dict()
                            elif isinstance(values_item_type_12_item_item_data, Document):
                                values_item_type_12_item_item = values_item_type_12_item_item_data.to_dict()
                            elif isinstance(values_item_type_12_item_item_data, FeedbackRatingDB):
                                values_item_type_12_item_item = values_item_type_12_item_item_data.to_dict()
                            elif isinstance(values_item_type_12_item_item_data, FeedbackAggregate):
                                values_item_type_12_item_item = values_item_type_12_item_item_data.to_dict()
                            else:
                                values_item_type_12_item_item = values_item_type_12_item_item_data
                            values_item_type_12_item.append(values_item_type_12_item_item)

                        values_item.append(values_item_type_12_item)

                elif isinstance(values_item_data, list):
                    values_item = []
                    for values_item_type_13_item_data in values_item_data:
                        values_item_type_13_item = []
                        for values_item_type_13_item_item_data in values_item_type_13_item_data:
                            values_item_type_13_item_item = []
                            for values_item_type_13_item_item_item_data in values_item_type_13_item_item_data:
                                values_item_type_13_item_item_item: Union[None, bool, dict[str, Any], float, int, str]
                                if isinstance(values_item_type_13_item_item_item_data, UUID):
                                    values_item_type_13_item_item_item = str(values_item_type_13_item_item_item_data)
                                elif isinstance(values_item_type_13_item_item_item_data, datetime.datetime):
                                    values_item_type_13_item_item_item = (
                                        values_item_type_13_item_item_item_data.isoformat()
                                    )
                                elif isinstance(values_item_type_13_item_item_item_data, Segment):
                                    values_item_type_13_item_item_item = (
                                        values_item_type_13_item_item_item_data.to_dict()
                                    )
                                elif isinstance(values_item_type_13_item_item_item_data, HallucinationSegment):
                                    values_item_type_13_item_item_item = (
                                        values_item_type_13_item_item_item_data.to_dict()
                                    )
                                elif isinstance(values_item_type_13_item_item_item_data, Document):
                                    values_item_type_13_item_item_item = (
                                        values_item_type_13_item_item_item_data.to_dict()
                                    )
                                elif isinstance(values_item_type_13_item_item_item_data, FeedbackRatingDB):
                                    values_item_type_13_item_item_item = (
                                        values_item_type_13_item_item_item_data.to_dict()
                                    )
                                elif isinstance(values_item_type_13_item_item_item_data, FeedbackAggregate):
                                    values_item_type_13_item_item_item = (
                                        values_item_type_13_item_item_item_data.to_dict()
                                    )
                                else:
                                    values_item_type_13_item_item_item = values_item_type_13_item_item_item_data
                                values_item_type_13_item_item.append(values_item_type_13_item_item_item)

                            values_item_type_13_item.append(values_item_type_13_item_item)

                        values_item.append(values_item_type_13_item)

                else:
                    values_item = values_item_data
                values.append(values_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"name": name})
        if alert is not UNSET:
            field_dict["alert"] = alert
        if data_type is not UNSET:
            field_dict["data_type"] = data_type
        if description is not UNSET:
            field_dict["description"] = description
        if display_values is not UNSET:
            field_dict["display_values"] = display_values
        if filterable is not UNSET:
            field_dict["filterable"] = filterable
        if group_description is not UNSET:
            field_dict["group_description"] = group_description
        if group_label is not UNSET:
            field_dict["group_label"] = group_label
        if group_name is not UNSET:
            field_dict["group_name"] = group_name
        if job_info is not UNSET:
            field_dict["job_info"] = job_info
        if job_type is not UNSET:
            field_dict["job_type"] = job_type
        if label is not UNSET:
            field_dict["label"] = label
        if metric_infos is not UNSET:
            field_dict["metric_infos"] = metric_infos
        if metric_threshold is not UNSET:
            field_dict["metric_threshold"] = metric_threshold
        if sortable is not UNSET:
            field_dict["sortable"] = sortable
        if values is not UNSET:
            field_dict["values"] = values

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.alert import Alert
        from ..models.document import Document
        from ..models.feedback_aggregate import FeedbackAggregate
        from ..models.feedback_rating_db import FeedbackRatingDB
        from ..models.hallucination_segment import HallucinationSegment
        from ..models.job_info import JobInfo
        from ..models.metric_computing import MetricComputing
        from ..models.metric_error import MetricError
        from ..models.metric_failed import MetricFailed
        from ..models.metric_not_applicable import MetricNotApplicable
        from ..models.metric_not_computed import MetricNotComputed
        from ..models.metric_pending import MetricPending
        from ..models.metric_success import MetricSuccess
        from ..models.metric_threshold import MetricThreshold
        from ..models.segment import Segment

        d = src_dict.copy()
        name = d.pop("name")

        def _parse_alert(data: object) -> Union["Alert", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                alert_type_0 = Alert.from_dict(data)

                return alert_type_0
            except:  # noqa: E722
                pass
            return cast(Union["Alert", None, Unset], data)

        alert = _parse_alert(d.pop("alert", UNSET))

        data_type = d.pop("data_type", UNSET)

        def _parse_description(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        description = _parse_description(d.pop("description", UNSET))

        def _parse_display_values(data: object) -> Union[None, Unset, list[Union[None, str]]]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                display_values_type_0 = []
                _display_values_type_0 = data
                for display_values_type_0_item_data in _display_values_type_0:

                    def _parse_display_values_type_0_item(data: object) -> Union[None, str]:
                        if data is None:
                            return data
                        return cast(Union[None, str], data)

                    display_values_type_0_item = _parse_display_values_type_0_item(display_values_type_0_item_data)

                    display_values_type_0.append(display_values_type_0_item)

                return display_values_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, list[Union[None, str]]], data)

        display_values = _parse_display_values(d.pop("display_values", UNSET))

        filterable = d.pop("filterable", UNSET)

        def _parse_group_description(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        group_description = _parse_group_description(d.pop("group_description", UNSET))

        def _parse_group_label(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        group_label = _parse_group_label(d.pop("group_label", UNSET))

        def _parse_group_name(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        group_name = _parse_group_name(d.pop("group_name", UNSET))

        def _parse_job_info(data: object) -> Union[None, Unset, list[Union["JobInfo", None]]]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                job_info_type_0 = []
                _job_info_type_0 = data
                for job_info_type_0_item_data in _job_info_type_0:

                    def _parse_job_info_type_0_item(data: object) -> Union["JobInfo", None]:
                        if data is None:
                            return data
                        try:
                            if not isinstance(data, dict):
                                raise TypeError()
                            job_info_type_0_item_type_0 = JobInfo.from_dict(data)

                            return job_info_type_0_item_type_0
                        except:  # noqa: E722
                            pass
                        return cast(Union["JobInfo", None], data)

                    job_info_type_0_item = _parse_job_info_type_0_item(job_info_type_0_item_data)

                    job_info_type_0.append(job_info_type_0_item)

                return job_info_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, list[Union["JobInfo", None]]], data)

        job_info = _parse_job_info(d.pop("job_info", UNSET))

        def _parse_job_type(data: object) -> Union[None, ScorerType, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                job_type_type_0 = ScorerType(data)

                return job_type_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, ScorerType, Unset], data)

        job_type = _parse_job_type(d.pop("job_type", UNSET))

        def _parse_label(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        label = _parse_label(d.pop("label", UNSET))

        metric_infos = []
        _metric_infos = d.pop("metric_infos", UNSET)
        for metric_infos_item_data in _metric_infos or []:

            def _parse_metric_infos_item(
                data: object,
            ) -> Union[
                "MetricComputing",
                "MetricError",
                "MetricFailed",
                "MetricNotApplicable",
                "MetricNotComputed",
                "MetricPending",
                "MetricSuccess",
            ]:
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    metric_infos_item_type_0 = MetricNotComputed.from_dict(data)

                    return metric_infos_item_type_0
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    metric_infos_item_type_1 = MetricPending.from_dict(data)

                    return metric_infos_item_type_1
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    metric_infos_item_type_2 = MetricComputing.from_dict(data)

                    return metric_infos_item_type_2
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    metric_infos_item_type_3 = MetricNotApplicable.from_dict(data)

                    return metric_infos_item_type_3
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    metric_infos_item_type_4 = MetricSuccess.from_dict(data)

                    return metric_infos_item_type_4
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    metric_infos_item_type_5 = MetricError.from_dict(data)

                    return metric_infos_item_type_5
                except:  # noqa: E722
                    pass
                if not isinstance(data, dict):
                    raise TypeError()
                metric_infos_item_type_6 = MetricFailed.from_dict(data)

                return metric_infos_item_type_6

            metric_infos_item = _parse_metric_infos_item(metric_infos_item_data)

            metric_infos.append(metric_infos_item)

        def _parse_metric_threshold(data: object) -> Union["MetricThreshold", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                metric_threshold_type_0 = MetricThreshold.from_dict(data)

                return metric_threshold_type_0
            except:  # noqa: E722
                pass
            return cast(Union["MetricThreshold", None, Unset], data)

        metric_threshold = _parse_metric_threshold(d.pop("metric_threshold", UNSET))

        sortable = d.pop("sortable", UNSET)

        values = []
        _values = d.pop("values", UNSET)
        for values_item_data in _values or []:

            def _parse_values_item(
                data: object,
            ) -> Union[
                "Document",
                "FeedbackAggregate",
                "FeedbackRatingDB",
                "HallucinationSegment",
                "Segment",
                None,
                UUID,
                bool,
                datetime.datetime,
                float,
                int,
                list[
                    Union[
                        "Document",
                        "FeedbackAggregate",
                        "FeedbackRatingDB",
                        "HallucinationSegment",
                        "Segment",
                        None,
                        UUID,
                        bool,
                        datetime.datetime,
                        float,
                        int,
                        str,
                    ]
                ],
                list[
                    list[
                        Union[
                            "Document",
                            "FeedbackAggregate",
                            "FeedbackRatingDB",
                            "HallucinationSegment",
                            "Segment",
                            None,
                            UUID,
                            bool,
                            datetime.datetime,
                            float,
                            int,
                            str,
                        ]
                    ]
                ],
                list[
                    list[
                        list[
                            Union[
                                "Document",
                                "FeedbackAggregate",
                                "FeedbackRatingDB",
                                "HallucinationSegment",
                                "Segment",
                                None,
                                UUID,
                                bool,
                                datetime.datetime,
                                float,
                                int,
                                str,
                            ]
                        ]
                    ]
                ],
                str,
            ]:
                if data is None:
                    return data
                try:
                    if not isinstance(data, str):
                        raise TypeError()
                    values_item_type_4 = UUID(data)

                    return values_item_type_4
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, str):
                        raise TypeError()
                    values_item_type_5 = isoparse(data)

                    return values_item_type_5
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    values_item_type_6 = Segment.from_dict(data)

                    return values_item_type_6
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    values_item_type_7 = HallucinationSegment.from_dict(data)

                    return values_item_type_7
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    values_item_type_8 = Document.from_dict(data)

                    return values_item_type_8
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    values_item_type_9 = FeedbackRatingDB.from_dict(data)

                    return values_item_type_9
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    values_item_type_10 = FeedbackAggregate.from_dict(data)

                    return values_item_type_10
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, list):
                        raise TypeError()
                    values_item_type_11 = []
                    _values_item_type_11 = data
                    for values_item_type_11_item_data in _values_item_type_11:

                        def _parse_values_item_type_11_item(
                            data: object,
                        ) -> Union[
                            "Document",
                            "FeedbackAggregate",
                            "FeedbackRatingDB",
                            "HallucinationSegment",
                            "Segment",
                            None,
                            UUID,
                            bool,
                            datetime.datetime,
                            float,
                            int,
                            str,
                        ]:
                            if data is None:
                                return data
                            try:
                                if not isinstance(data, str):
                                    raise TypeError()
                                values_item_type_11_item_type_4 = UUID(data)

                                return values_item_type_11_item_type_4
                            except:  # noqa: E722
                                pass
                            try:
                                if not isinstance(data, str):
                                    raise TypeError()
                                values_item_type_11_item_type_5 = isoparse(data)

                                return values_item_type_11_item_type_5
                            except:  # noqa: E722
                                pass
                            try:
                                if not isinstance(data, dict):
                                    raise TypeError()
                                values_item_type_11_item_type_6 = Segment.from_dict(data)

                                return values_item_type_11_item_type_6
                            except:  # noqa: E722
                                pass
                            try:
                                if not isinstance(data, dict):
                                    raise TypeError()
                                values_item_type_11_item_type_7 = HallucinationSegment.from_dict(data)

                                return values_item_type_11_item_type_7
                            except:  # noqa: E722
                                pass
                            try:
                                if not isinstance(data, dict):
                                    raise TypeError()
                                values_item_type_11_item_type_8 = Document.from_dict(data)

                                return values_item_type_11_item_type_8
                            except:  # noqa: E722
                                pass
                            try:
                                if not isinstance(data, dict):
                                    raise TypeError()
                                values_item_type_11_item_type_9 = FeedbackRatingDB.from_dict(data)

                                return values_item_type_11_item_type_9
                            except:  # noqa: E722
                                pass
                            try:
                                if not isinstance(data, dict):
                                    raise TypeError()
                                values_item_type_11_item_type_10 = FeedbackAggregate.from_dict(data)

                                return values_item_type_11_item_type_10
                            except:  # noqa: E722
                                pass
                            return cast(
                                Union[
                                    "Document",
                                    "FeedbackAggregate",
                                    "FeedbackRatingDB",
                                    "HallucinationSegment",
                                    "Segment",
                                    None,
                                    UUID,
                                    bool,
                                    datetime.datetime,
                                    float,
                                    int,
                                    str,
                                ],
                                data,
                            )

                        values_item_type_11_item = _parse_values_item_type_11_item(values_item_type_11_item_data)

                        values_item_type_11.append(values_item_type_11_item)

                    return values_item_type_11
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, list):
                        raise TypeError()
                    values_item_type_12 = []
                    _values_item_type_12 = data
                    for values_item_type_12_item_data in _values_item_type_12:
                        values_item_type_12_item = []
                        _values_item_type_12_item = values_item_type_12_item_data
                        for values_item_type_12_item_item_data in _values_item_type_12_item:

                            def _parse_values_item_type_12_item_item(
                                data: object,
                            ) -> Union[
                                "Document",
                                "FeedbackAggregate",
                                "FeedbackRatingDB",
                                "HallucinationSegment",
                                "Segment",
                                None,
                                UUID,
                                bool,
                                datetime.datetime,
                                float,
                                int,
                                str,
                            ]:
                                if data is None:
                                    return data
                                try:
                                    if not isinstance(data, str):
                                        raise TypeError()
                                    values_item_type_12_item_item_type_4 = UUID(data)

                                    return values_item_type_12_item_item_type_4
                                except:  # noqa: E722
                                    pass
                                try:
                                    if not isinstance(data, str):
                                        raise TypeError()
                                    values_item_type_12_item_item_type_5 = isoparse(data)

                                    return values_item_type_12_item_item_type_5
                                except:  # noqa: E722
                                    pass
                                try:
                                    if not isinstance(data, dict):
                                        raise TypeError()
                                    values_item_type_12_item_item_type_6 = Segment.from_dict(data)

                                    return values_item_type_12_item_item_type_6
                                except:  # noqa: E722
                                    pass
                                try:
                                    if not isinstance(data, dict):
                                        raise TypeError()
                                    values_item_type_12_item_item_type_7 = HallucinationSegment.from_dict(data)

                                    return values_item_type_12_item_item_type_7
                                except:  # noqa: E722
                                    pass
                                try:
                                    if not isinstance(data, dict):
                                        raise TypeError()
                                    values_item_type_12_item_item_type_8 = Document.from_dict(data)

                                    return values_item_type_12_item_item_type_8
                                except:  # noqa: E722
                                    pass
                                try:
                                    if not isinstance(data, dict):
                                        raise TypeError()
                                    values_item_type_12_item_item_type_9 = FeedbackRatingDB.from_dict(data)

                                    return values_item_type_12_item_item_type_9
                                except:  # noqa: E722
                                    pass
                                try:
                                    if not isinstance(data, dict):
                                        raise TypeError()
                                    values_item_type_12_item_item_type_10 = FeedbackAggregate.from_dict(data)

                                    return values_item_type_12_item_item_type_10
                                except:  # noqa: E722
                                    pass
                                return cast(
                                    Union[
                                        "Document",
                                        "FeedbackAggregate",
                                        "FeedbackRatingDB",
                                        "HallucinationSegment",
                                        "Segment",
                                        None,
                                        UUID,
                                        bool,
                                        datetime.datetime,
                                        float,
                                        int,
                                        str,
                                    ],
                                    data,
                                )

                            values_item_type_12_item_item = _parse_values_item_type_12_item_item(
                                values_item_type_12_item_item_data
                            )

                            values_item_type_12_item.append(values_item_type_12_item_item)

                        values_item_type_12.append(values_item_type_12_item)

                    return values_item_type_12
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, list):
                        raise TypeError()
                    values_item_type_13 = []
                    _values_item_type_13 = data
                    for values_item_type_13_item_data in _values_item_type_13:
                        values_item_type_13_item = []
                        _values_item_type_13_item = values_item_type_13_item_data
                        for values_item_type_13_item_item_data in _values_item_type_13_item:
                            values_item_type_13_item_item = []
                            _values_item_type_13_item_item = values_item_type_13_item_item_data
                            for values_item_type_13_item_item_item_data in _values_item_type_13_item_item:

                                def _parse_values_item_type_13_item_item_item(
                                    data: object,
                                ) -> Union[
                                    "Document",
                                    "FeedbackAggregate",
                                    "FeedbackRatingDB",
                                    "HallucinationSegment",
                                    "Segment",
                                    None,
                                    UUID,
                                    bool,
                                    datetime.datetime,
                                    float,
                                    int,
                                    str,
                                ]:
                                    if data is None:
                                        return data
                                    try:
                                        if not isinstance(data, str):
                                            raise TypeError()
                                        values_item_type_13_item_item_item_type_4 = UUID(data)

                                        return values_item_type_13_item_item_item_type_4
                                    except:  # noqa: E722
                                        pass
                                    try:
                                        if not isinstance(data, str):
                                            raise TypeError()
                                        values_item_type_13_item_item_item_type_5 = isoparse(data)

                                        return values_item_type_13_item_item_item_type_5
                                    except:  # noqa: E722
                                        pass
                                    try:
                                        if not isinstance(data, dict):
                                            raise TypeError()
                                        values_item_type_13_item_item_item_type_6 = Segment.from_dict(data)

                                        return values_item_type_13_item_item_item_type_6
                                    except:  # noqa: E722
                                        pass
                                    try:
                                        if not isinstance(data, dict):
                                            raise TypeError()
                                        values_item_type_13_item_item_item_type_7 = HallucinationSegment.from_dict(data)

                                        return values_item_type_13_item_item_item_type_7
                                    except:  # noqa: E722
                                        pass
                                    try:
                                        if not isinstance(data, dict):
                                            raise TypeError()
                                        values_item_type_13_item_item_item_type_8 = Document.from_dict(data)

                                        return values_item_type_13_item_item_item_type_8
                                    except:  # noqa: E722
                                        pass
                                    try:
                                        if not isinstance(data, dict):
                                            raise TypeError()
                                        values_item_type_13_item_item_item_type_9 = FeedbackRatingDB.from_dict(data)

                                        return values_item_type_13_item_item_item_type_9
                                    except:  # noqa: E722
                                        pass
                                    try:
                                        if not isinstance(data, dict):
                                            raise TypeError()
                                        values_item_type_13_item_item_item_type_10 = FeedbackAggregate.from_dict(data)

                                        return values_item_type_13_item_item_item_type_10
                                    except:  # noqa: E722
                                        pass
                                    return cast(
                                        Union[
                                            "Document",
                                            "FeedbackAggregate",
                                            "FeedbackRatingDB",
                                            "HallucinationSegment",
                                            "Segment",
                                            None,
                                            UUID,
                                            bool,
                                            datetime.datetime,
                                            float,
                                            int,
                                            str,
                                        ],
                                        data,
                                    )

                                values_item_type_13_item_item_item = _parse_values_item_type_13_item_item_item(
                                    values_item_type_13_item_item_item_data
                                )

                                values_item_type_13_item_item.append(values_item_type_13_item_item_item)

                            values_item_type_13_item.append(values_item_type_13_item_item)

                        values_item_type_13.append(values_item_type_13_item)

                    return values_item_type_13
                except:  # noqa: E722
                    pass
                return cast(
                    Union[
                        "Document",
                        "FeedbackAggregate",
                        "FeedbackRatingDB",
                        "HallucinationSegment",
                        "Segment",
                        None,
                        UUID,
                        bool,
                        datetime.datetime,
                        float,
                        int,
                        list[
                            Union[
                                "Document",
                                "FeedbackAggregate",
                                "FeedbackRatingDB",
                                "HallucinationSegment",
                                "Segment",
                                None,
                                UUID,
                                bool,
                                datetime.datetime,
                                float,
                                int,
                                str,
                            ]
                        ],
                        list[
                            list[
                                Union[
                                    "Document",
                                    "FeedbackAggregate",
                                    "FeedbackRatingDB",
                                    "HallucinationSegment",
                                    "Segment",
                                    None,
                                    UUID,
                                    bool,
                                    datetime.datetime,
                                    float,
                                    int,
                                    str,
                                ]
                            ]
                        ],
                        list[
                            list[
                                list[
                                    Union[
                                        "Document",
                                        "FeedbackAggregate",
                                        "FeedbackRatingDB",
                                        "HallucinationSegment",
                                        "Segment",
                                        None,
                                        UUID,
                                        bool,
                                        datetime.datetime,
                                        float,
                                        int,
                                        str,
                                    ]
                                ]
                            ]
                        ],
                        str,
                    ],
                    data,
                )

            values_item = _parse_values_item(values_item_data)

            values.append(values_item)

        prompt_run_column = cls(
            name=name,
            alert=alert,
            data_type=data_type,
            description=description,
            display_values=display_values,
            filterable=filterable,
            group_description=group_description,
            group_label=group_label,
            group_name=group_name,
            job_info=job_info,
            job_type=job_type,
            label=label,
            metric_infos=metric_infos,
            metric_threshold=metric_threshold,
            sortable=sortable,
            values=values,
        )

        prompt_run_column.additional_properties = d
        return prompt_run_column

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.target_labels import TargetLabels


T = TypeVar("T", bound="LabelCoOccurences")


@_attrs_define
class LabelCoOccurences:
    """Structure for each label co-occurrence response.

    Attributes:
        ids (list[int]):
        labels (list['TargetLabels']):
        starting_label (str):
    """

    ids: list[int]
    labels: list["TargetLabels"]
    starting_label: str
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        ids = self.ids

        labels = []
        for labels_item_data in self.labels:
            labels_item = labels_item_data.to_dict()
            labels.append(labels_item)

        starting_label = self.starting_label

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"ids": ids, "labels": labels, "starting_label": starting_label})

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.target_labels import TargetLabels

        d = src_dict.copy()
        ids = cast(list[int], d.pop("ids"))

        labels = []
        _labels = d.pop("labels")
        for labels_item_data in _labels:
            labels_item = TargetLabels.from_dict(labels_item_data)

            labels.append(labels_item)

        starting_label = d.pop("starting_label")

        label_co_occurences = cls(ids=ids, labels=labels, starting_label=starting_label)

        label_co_occurences.additional_properties = d
        return label_co_occurences

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.http_validation_error import HTTPValidationError
from ...models.prompt_sample_i_ds import PromptSampleIDs
from ...types import UNSET, Response


def _get_kwargs(project_id: str, *, run_ids: list[str]) -> dict[str, Any]:
    params: dict[str, Any] = {}

    json_run_ids = run_ids

    params["run_ids"] = json_run_ids

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": f"/projects/{project_id}/runs/prompts/rows/columnar",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[HTTPValidationError, PromptSampleIDs]]:
    if response.status_code == 200:
        response_200 = PromptSampleIDs.from_dict(response.json())

        return response_200
    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[HTTPValidationError, PromptSampleIDs]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    project_id: str, *, client: AuthenticatedClient, run_ids: list[str]
) -> Response[Union[HTTPValidationError, PromptSampleIDs]]:
    """Get Multi Run Sample Ids

     Given a list of runs with the same inputs, output the indices of each sample in each run.

    Args:
        project_id (str):
        run_ids (list[str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[HTTPValidationError, PromptSampleIDs]]
    """

    kwargs = _get_kwargs(project_id=project_id, run_ids=run_ids)

    response = client.get_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


def sync(
    project_id: str, *, client: AuthenticatedClient, run_ids: list[str]
) -> Optional[Union[HTTPValidationError, PromptSampleIDs]]:
    """Get Multi Run Sample Ids

     Given a list of runs with the same inputs, output the indices of each sample in each run.

    Args:
        project_id (str):
        run_ids (list[str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[HTTPValidationError, PromptSampleIDs]
    """

    return sync_detailed(project_id=project_id, client=client, run_ids=run_ids).parsed


async def asyncio_detailed(
    project_id: str, *, client: AuthenticatedClient, run_ids: list[str]
) -> Response[Union[HTTPValidationError, PromptSampleIDs]]:
    """Get Multi Run Sample Ids

     Given a list of runs with the same inputs, output the indices of each sample in each run.

    Args:
        project_id (str):
        run_ids (list[str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[HTTPValidationError, PromptSampleIDs]]
    """

    kwargs = _get_kwargs(project_id=project_id, run_ids=run_ids)

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    project_id: str, *, client: AuthenticatedClient, run_ids: list[str]
) -> Optional[Union[HTTPValidationError, PromptSampleIDs]]:
    """Get Multi Run Sample Ids

     Given a list of runs with the same inputs, output the indices of each sample in each run.

    Args:
        project_id (str):
        run_ids (list[str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[HTTPValidationError, PromptSampleIDs]
    """

    return (await asyncio_detailed(project_id=project_id, client=client, run_ids=run_ids)).parsed



from http import HTTPStatus
from typing import Any, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.body_update_prompt_dataset_projects_project_id_prompt_datasets_dataset_id_put import (
    BodyUpdatePromptDatasetProjectsProjectIdPromptDatasetsDatasetIdPut,
)
from ...models.dataset_format import DatasetFormat
from ...models.http_validation_error import HTTPValidationError
from ...models.prompt_dataset_db import PromptDatasetDB
from ...types import UNSET, Response, Unset


def _get_kwargs(
    project_id: str,
    dataset_id: str,
    *,
    body: BodyUpdatePromptDatasetProjectsProjectIdPromptDatasetsDatasetIdPut,
    file_name: Union[None, Unset, str] = UNSET,
    num_rows: Union[None, Unset, int] = UNSET,
    format_: Union[Unset, DatasetFormat] = UNSET,
) -> dict[str, Any]:
    headers: dict[str, Any] = {}

    params: dict[str, Any] = {}

    json_file_name: Union[None, Unset, str]
    if isinstance(file_name, Unset):
        json_file_name = UNSET
    else:
        json_file_name = file_name
    params["file_name"] = json_file_name

    json_num_rows: Union[None, Unset, int]
    if isinstance(num_rows, Unset):
        json_num_rows = UNSET
    else:
        json_num_rows = num_rows
    params["num_rows"] = json_num_rows

    json_format_: Union[Unset, str] = UNSET
    if not isinstance(format_, Unset):
        json_format_ = format_.value

    params["format"] = json_format_

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "put",
        "url": f"/projects/{project_id}/prompt_datasets/{dataset_id}",
        "params": params,
    }

    _body = body.to_multipart()

    _kwargs["files"] = _body

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[HTTPValidationError, PromptDatasetDB]]:
    if response.status_code == 200:
        response_200 = PromptDatasetDB.from_dict(response.json())

        return response_200
    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[HTTPValidationError, PromptDatasetDB]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    project_id: str,
    dataset_id: str,
    *,
    client: AuthenticatedClient,
    body: BodyUpdatePromptDatasetProjectsProjectIdPromptDatasetsDatasetIdPut,
    file_name: Union[None, Unset, str] = UNSET,
    num_rows: Union[None, Unset, int] = UNSET,
    format_: Union[Unset, DatasetFormat] = UNSET,
) -> Response[Union[HTTPValidationError, PromptDatasetDB]]:
    """Update Prompt Dataset

    Args:
        project_id (str):
        dataset_id (str):
        file_name (Union[None, Unset, str]):
        num_rows (Union[None, Unset, int]):
        format_ (Union[Unset, DatasetFormat]):
        body (BodyUpdatePromptDatasetProjectsProjectIdPromptDatasetsDatasetIdPut):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[HTTPValidationError, PromptDatasetDB]]
    """

    kwargs = _get_kwargs(
        project_id=project_id, dataset_id=dataset_id, body=body, file_name=file_name, num_rows=num_rows, format_=format_
    )

    response = client.get_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


def sync(
    project_id: str,
    dataset_id: str,
    *,
    client: AuthenticatedClient,
    body: BodyUpdatePromptDatasetProjectsProjectIdPromptDatasetsDatasetIdPut,
    file_name: Union[None, Unset, str] = UNSET,
    num_rows: Union[None, Unset, int] = UNSET,
    format_: Union[Unset, DatasetFormat] = UNSET,
) -> Optional[Union[HTTPValidationError, PromptDatasetDB]]:
    """Update Prompt Dataset

    Args:
        project_id (str):
        dataset_id (str):
        file_name (Union[None, Unset, str]):
        num_rows (Union[None, Unset, int]):
        format_ (Union[Unset, DatasetFormat]):
        body (BodyUpdatePromptDatasetProjectsProjectIdPromptDatasetsDatasetIdPut):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[HTTPValidationError, PromptDatasetDB]
    """

    return sync_detailed(
        project_id=project_id,
        dataset_id=dataset_id,
        client=client,
        body=body,
        file_name=file_name,
        num_rows=num_rows,
        format_=format_,
    ).parsed


async def asyncio_detailed(
    project_id: str,
    dataset_id: str,
    *,
    client: AuthenticatedClient,
    body: BodyUpdatePromptDatasetProjectsProjectIdPromptDatasetsDatasetIdPut,
    file_name: Union[None, Unset, str] = UNSET,
    num_rows: Union[None, Unset, int] = UNSET,
    format_: Union[Unset, DatasetFormat] = UNSET,
) -> Response[Union[HTTPValidationError, PromptDatasetDB]]:
    """Update Prompt Dataset

    Args:
        project_id (str):
        dataset_id (str):
        file_name (Union[None, Unset, str]):
        num_rows (Union[None, Unset, int]):
        format_ (Union[Unset, DatasetFormat]):
        body (BodyUpdatePromptDatasetProjectsProjectIdPromptDatasetsDatasetIdPut):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[HTTPValidationError, PromptDatasetDB]]
    """

    kwargs = _get_kwargs(
        project_id=project_id, dataset_id=dataset_id, body=body, file_name=file_name, num_rows=num_rows, format_=format_
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    project_id: str,
    dataset_id: str,
    *,
    client: AuthenticatedClient,
    body: BodyUpdatePromptDatasetProjectsProjectIdPromptDatasetsDatasetIdPut,
    file_name: Union[None, Unset, str] = UNSET,
    num_rows: Union[None, Unset, int] = UNSET,
    format_: Union[Unset, DatasetFormat] = UNSET,
) -> Optional[Union[HTTPValidationError, PromptDatasetDB]]:
    """Update Prompt Dataset

    Args:
        project_id (str):
        dataset_id (str):
        file_name (Union[None, Unset, str]):
        num_rows (Union[None, Unset, int]):
        format_ (Union[Unset, DatasetFormat]):
        body (BodyUpdatePromptDatasetProjectsProjectIdPromptDatasetsDatasetIdPut):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[HTTPValidationError, PromptDatasetDB]
    """

    return (
        await asyncio_detailed(
            project_id=project_id,
            dataset_id=dataset_id,
            client=client,
            body=body,
            file_name=file_name,
            num_rows=num_rows,
            format_=format_,
        )
    ).parsed



from enum import Enum


class TransactionRecordStatus(str, Enum):
    PROCESSED = "processed"
    PROCESSING = "processing"
    UNPROCESSED = "unprocessed"

    def __str__(self) -> str:
        return str(self.value)



from http import HTTPStatus
from typing import Any, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.http_validation_error import HTTPValidationError
from ...models.remote_edit_export_request import RemoteEditExportRequest
from ...models.remote_edit_export_response import RemoteEditExportResponse
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *, body: RemoteEditExportRequest, ignore_conflicting_edits: Union[Unset, bool] = True
) -> dict[str, Any]:
    headers: dict[str, Any] = {}

    params: dict[str, Any] = {}

    params["ignore_conflicting_edits"] = ignore_conflicting_edits

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {"method": "post", "url": "/edits/export/remote", "params": params}

    _body = body.to_dict()

    _kwargs["json"] = _body
    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[HTTPValidationError, RemoteEditExportResponse]]:
    if response.status_code == 200:
        response_200 = RemoteEditExportResponse.from_dict(response.json())

        return response_200
    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[HTTPValidationError, RemoteEditExportResponse]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *, client: AuthenticatedClient, body: RemoteEditExportRequest, ignore_conflicting_edits: Union[Unset, bool] = True
) -> Response[Union[HTTPValidationError, RemoteEditExportResponse]]:
    """Export Edits Remote

     Exports the resulting dataframe after a series of edits to a remote object store.

    Note that the file type is determined by the suffix of the object name. Must be one of csv, parquet,
    json, arrow

    If overrides are provided, and the original edit had a task, the override MUST explicitly have a
    task set (typically
    the same task as the edit)

    Args:
        ignore_conflicting_edits (Union[Unset, bool]):  Default: True.
        body (RemoteEditExportRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[HTTPValidationError, RemoteEditExportResponse]]
    """

    kwargs = _get_kwargs(body=body, ignore_conflicting_edits=ignore_conflicting_edits)

    response = client.get_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


def sync(
    *, client: AuthenticatedClient, body: RemoteEditExportRequest, ignore_conflicting_edits: Union[Unset, bool] = True
) -> Optional[Union[HTTPValidationError, RemoteEditExportResponse]]:
    """Export Edits Remote

     Exports the resulting dataframe after a series of edits to a remote object store.

    Note that the file type is determined by the suffix of the object name. Must be one of csv, parquet,
    json, arrow

    If overrides are provided, and the original edit had a task, the override MUST explicitly have a
    task set (typically
    the same task as the edit)

    Args:
        ignore_conflicting_edits (Union[Unset, bool]):  Default: True.
        body (RemoteEditExportRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[HTTPValidationError, RemoteEditExportResponse]
    """

    return sync_detailed(client=client, body=body, ignore_conflicting_edits=ignore_conflicting_edits).parsed


async def asyncio_detailed(
    *, client: AuthenticatedClient, body: RemoteEditExportRequest, ignore_conflicting_edits: Union[Unset, bool] = True
) -> Response[Union[HTTPValidationError, RemoteEditExportResponse]]:
    """Export Edits Remote

     Exports the resulting dataframe after a series of edits to a remote object store.

    Note that the file type is determined by the suffix of the object name. Must be one of csv, parquet,
    json, arrow

    If overrides are provided, and the original edit had a task, the override MUST explicitly have a
    task set (typically
    the same task as the edit)

    Args:
        ignore_conflicting_edits (Union[Unset, bool]):  Default: True.
        body (RemoteEditExportRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[HTTPValidationError, RemoteEditExportResponse]]
    """

    kwargs = _get_kwargs(body=body, ignore_conflicting_edits=ignore_conflicting_edits)

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *, client: AuthenticatedClient, body: RemoteEditExportRequest, ignore_conflicting_edits: Union[Unset, bool] = True
) -> Optional[Union[HTTPValidationError, RemoteEditExportResponse]]:
    """Export Edits Remote

     Exports the resulting dataframe after a series of edits to a remote object store.

    Note that the file type is determined by the suffix of the object name. Must be one of csv, parquet,
    json, arrow

    If overrides are provided, and the original edit had a task, the override MUST explicitly have a
    task set (typically
    the same task as the edit)

    Args:
        ignore_conflicting_edits (Union[Unset, bool]):  Default: True.
        body (RemoteEditExportRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[HTTPValidationError, RemoteEditExportResponse]
    """

    return (await asyncio_detailed(client=client, body=body, ignore_conflicting_edits=ignore_conflicting_edits)).parsed



from typing import Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="MetaFilter")


@_attrs_define
class MetaFilter:
    """A class for filtering arbitrary metadata columns.

    Attributes:
        name (str):
        greater_than (Union[None, Unset, float]):
        is_equal (Union[None, Unset, float, int]):
        isin (Union[None, Unset, list[Union[None, bool, int, str]]]):
        less_than (Union[None, Unset, float]):
    """

    name: str
    greater_than: Union[None, Unset, float] = UNSET
    is_equal: Union[None, Unset, float, int] = UNSET
    isin: Union[None, Unset, list[Union[None, bool, int, str]]] = UNSET
    less_than: Union[None, Unset, float] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        greater_than: Union[None, Unset, float]
        if isinstance(self.greater_than, Unset):
            greater_than = UNSET
        else:
            greater_than = self.greater_than

        is_equal: Union[None, Unset, float, int]
        if isinstance(self.is_equal, Unset):
            is_equal = UNSET
        else:
            is_equal = self.is_equal

        isin: Union[None, Unset, list[Union[None, bool, int, str]]]
        if isinstance(self.isin, Unset):
            isin = UNSET
        elif isinstance(self.isin, list):
            isin = []
            for isin_type_0_item_data in self.isin:
                isin_type_0_item: Union[None, bool, int, str]
                isin_type_0_item = isin_type_0_item_data
                isin.append(isin_type_0_item)

        else:
            isin = self.isin

        less_than: Union[None, Unset, float]
        if isinstance(self.less_than, Unset):
            less_than = UNSET
        else:
            less_than = self.less_than

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"name": name})
        if greater_than is not UNSET:
            field_dict["greater_than"] = greater_than
        if is_equal is not UNSET:
            field_dict["is_equal"] = is_equal
        if isin is not UNSET:
            field_dict["isin"] = isin
        if less_than is not UNSET:
            field_dict["less_than"] = less_than

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        def _parse_greater_than(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        greater_than = _parse_greater_than(d.pop("greater_than", UNSET))

        def _parse_is_equal(data: object) -> Union[None, Unset, float, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float, int], data)

        is_equal = _parse_is_equal(d.pop("is_equal", UNSET))

        def _parse_isin(data: object) -> Union[None, Unset, list[Union[None, bool, int, str]]]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                isin_type_0 = []
                _isin_type_0 = data
                for isin_type_0_item_data in _isin_type_0:

                    def _parse_isin_type_0_item(data: object) -> Union[None, bool, int, str]:
                        if data is None:
                            return data
                        return cast(Union[None, bool, int, str], data)

                    isin_type_0_item = _parse_isin_type_0_item(isin_type_0_item_data)

                    isin_type_0.append(isin_type_0_item)

                return isin_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, list[Union[None, bool, int, str]]], data)

        isin = _parse_isin(d.pop("isin", UNSET))

        def _parse_less_than(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        less_than = _parse_less_than(d.pop("less_than", UNSET))

        meta_filter = cls(name=name, greater_than=greater_than, is_equal=is_equal, isin=isin, less_than=less_than)

        meta_filter.additional_properties = d
        return meta_filter

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.base_prompt_template_version_response import BasePromptTemplateVersionResponse


T = TypeVar("T", bound="BasePromptTemplateResponse")


@_attrs_define
class BasePromptTemplateResponse:
    """Response from API to get a prompt template version.

    Attributes:
        all_available_versions (list[int]):
        id (str):
        max_version (int):
        name (str):
        selected_version (BasePromptTemplateVersionResponse): Base response from API for a prompt template version.
        selected_version_id (str):
        template (str):
        total_versions (int):
        all_versions (Union[Unset, list['BasePromptTemplateVersionResponse']]):
    """

    all_available_versions: list[int]
    id: str
    max_version: int
    name: str
    selected_version: "BasePromptTemplateVersionResponse"
    selected_version_id: str
    template: str
    total_versions: int
    all_versions: Union[Unset, list["BasePromptTemplateVersionResponse"]] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        all_available_versions = self.all_available_versions

        id = self.id

        max_version = self.max_version

        name = self.name

        selected_version = self.selected_version.to_dict()

        selected_version_id = self.selected_version_id

        template = self.template

        total_versions = self.total_versions

        all_versions: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.all_versions, Unset):
            all_versions = []
            for all_versions_item_data in self.all_versions:
                all_versions_item = all_versions_item_data.to_dict()
                all_versions.append(all_versions_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "all_available_versions": all_available_versions,
                "id": id,
                "max_version": max_version,
                "name": name,
                "selected_version": selected_version,
                "selected_version_id": selected_version_id,
                "template": template,
                "total_versions": total_versions,
            }
        )
        if all_versions is not UNSET:
            field_dict["all_versions"] = all_versions

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.base_prompt_template_version_response import BasePromptTemplateVersionResponse

        d = src_dict.copy()
        all_available_versions = cast(list[int], d.pop("all_available_versions"))

        id = d.pop("id")

        max_version = d.pop("max_version")

        name = d.pop("name")

        selected_version = BasePromptTemplateVersionResponse.from_dict(d.pop("selected_version"))

        selected_version_id = d.pop("selected_version_id")

        template = d.pop("template")

        total_versions = d.pop("total_versions")

        all_versions = []
        _all_versions = d.pop("all_versions", UNSET)
        for all_versions_item_data in _all_versions or []:
            all_versions_item = BasePromptTemplateVersionResponse.from_dict(all_versions_item_data)

            all_versions.append(all_versions_item)

        base_prompt_template_response = cls(
            all_available_versions=all_available_versions,
            id=id,
            max_version=max_version,
            name=name,
            selected_version=selected_version,
            selected_version_id=selected_version_id,
            template=template,
            total_versions=total_versions,
            all_versions=all_versions,
        )

        base_prompt_template_response.additional_properties = d
        return base_prompt_template_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.filter_options_response_filter_options import FilterOptionsResponseFilterOptions


T = TypeVar("T", bound="FilterOptionsResponse")


@_attrs_define
class FilterOptionsResponse:
    """
    Attributes:
        filter_options (Union[Unset, FilterOptionsResponseFilterOptions]):
    """

    filter_options: Union[Unset, "FilterOptionsResponseFilterOptions"] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        filter_options: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.filter_options, Unset):
            filter_options = self.filter_options.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if filter_options is not UNSET:
            field_dict["filter_options"] = filter_options

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.filter_options_response_filter_options import FilterOptionsResponseFilterOptions

        d = src_dict.copy()
        _filter_options = d.pop("filter_options", UNSET)
        filter_options: Union[Unset, FilterOptionsResponseFilterOptions]
        if isinstance(_filter_options, Unset):
            filter_options = UNSET
        else:
            filter_options = FilterOptionsResponseFilterOptions.from_dict(_filter_options)

        filter_options_response = cls(filter_options=filter_options)

        filter_options_response.additional_properties = d
        return filter_options_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from os import getenv
from typing import Optional

import httpx
from attrs import define, evolve, field

from galileo.constants import DEFAULT_API_URL
from galileo.resources.client import AuthenticatedClient


@define
class GalileoApiClient(AuthenticatedClient):
    """A Client which has been authenticated for use on secured endpoints

    The following are accepted as keyword arguments and will be used to construct httpx Clients internally:

        ``base_url``: The base URL for the API, all requests are made to a relative path to this URL
            This can also be set via the GALILEO_CONSOLE_URL environment variable

        ``api_key``: The API key to be sent with every request
            This can also be set via the GALILEO_API_KEY environment variable

        ``cookies``: A dictionary of cookies to be sent with every request

        ``headers``: A dictionary of headers to be sent with every request

        ``timeout``: The maximum amount of a time a request can take. API functions will raise
        httpx.TimeoutException if this is exceeded.

        ``verify_ssl``: Whether or not to verify the SSL certificate of the API server. This should be True in production,
        but can be set to False for testing purposes.

        ``follow_redirects``: Whether or not to follow redirects. Default value is False.

        ``httpx_args``: A dictionary of additional arguments to be passed to the ``httpx.Client`` and ``httpx.AsyncClient`` constructor.

    Attributes:
        raise_on_unexpected_status: Whether or not to raise an errors.UnexpectedStatus if the API returns a
            status code that was not documented in the source OpenAPI document. Can also be provided as a keyword
            argument to the constructor.
        token: The token to use for authentication
        prefix: The prefix to use for the Authorization header
        auth_header_name: The name of the Authorization header
    """

    _base_url: Optional[str] = field(factory=lambda: GalileoApiClient.get_api_url(), kw_only=True, alias="base_url")
    _api_key: Optional[str] = field(factory=lambda: getenv("GALILEO_API_KEY", None), kw_only=True, alias="api_key")
    token: Optional[str] = None

    api_key_header_name: str = "Galileo-API-Key"
    client_type_header_name: str = "client-type"
    client_type_header_value: str = "sdk-python"

    @staticmethod
    def get_console_url() -> str:
        console_url = getenv("GALILEO_CONSOLE_URL", DEFAULT_API_URL)
        if DEFAULT_API_URL == console_url:
            return "https://app.galileo.ai"

        return console_url

    def with_api_key(self, api_key: str) -> "GalileoApiClient":
        """Get a new client matching this one with a new API key"""
        if self._client is not None:
            self._client.headers.update({self.api_key_header_name: api_key})
        if self._async_client is not None:
            self._async_client.headers.update({self.api_key_header_name: api_key})
        return evolve(self, api_key=api_key)

    def get_httpx_client(self) -> httpx.Client:
        """Get the underlying httpx.Client, constructing a new one if not previously set"""
        if self._client is None:
            if self._api_key:
                self._headers[self.api_key_header_name] = self._api_key
            elif self.token:
                self._headers[self.auth_header_name] = f"{self.prefix} {self.token}" if self.prefix else self.token
            else:
                raise ValueError("Either api_key or token must be set")

            if not self._base_url:
                raise ValueError("base_url must be set")

            self._headers[self.client_type_header_name] = self.client_type_header_value

            self._client = httpx.Client(
                base_url=self._base_url,
                cookies=self._cookies,
                headers=self._headers,
                timeout=self._timeout,
                verify=self._verify_ssl,
                follow_redirects=self._follow_redirects,
                **self._httpx_args,
            )
        return self._client

    def get_async_httpx_client(self) -> httpx.AsyncClient:
        """Get the underlying httpx.AsyncClient, constructing a new one if not previously set"""
        if self._async_client is None:
            if self._api_key:
                self._headers[self.api_key_header_name] = self._api_key
            elif self.token:
                self._headers[self.auth_header_name] = f"{self.prefix} {self.token}" if self.prefix else self.token
            else:
                raise ValueError("Either api_key or token must be set")

            if not self._base_url:
                raise ValueError("base_url must be set")

            self._headers[self.client_type_header_name] = self.client_type_header_value

            self._async_client = httpx.AsyncClient(
                base_url=self._base_url,
                cookies=self._cookies,
                headers=self._headers,
                timeout=self._timeout,
                verify=self._verify_ssl,
                follow_redirects=self._follow_redirects,
                **self._httpx_args,
            )
        return self._async_client

    @staticmethod
    def get_api_url(base_url: Optional[str] = None) -> str:
        api_url = base_url or getenv("GALILEO_CONSOLE_URL", DEFAULT_API_URL)
        if api_url is None:
            raise ValueError("base_url or GALILEO_CONSOLE_URL must be set")
        if any(map(api_url.__contains__, ["localhost", "127.0.0.1"])):
            api_url = "http://localhost:8088"
        else:
            api_url = api_url.replace("app.galileo.ai", "api.galileo.ai").replace("console", "api")
        return api_url



from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.alert_importance import AlertImportance
from ..models.x_ray_class import XRayClass
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.extra_alert_filters import ExtraAlertFilters
    from ..models.filter_params import FilterParams
    from ..models.x_ray_card_db_message_fields import XRayCardDBMessageFields


T = TypeVar("T", bound="XRayCardDB")


@_attrs_define
class XRayCardDB:
    """
    Attributes:
        alert_id (int):
        dismissed (bool):
        id (str):
        message_fields (XRayCardDBMessageFields):
        xray_class (XRayClass):
        extra_filters (Union['ExtraAlertFilters', None, Unset]):
        importance (Union[AlertImportance, None, Unset]):
        logic (Union['FilterParams', None, Unset]):
        message (Union[Unset, str]):  Default: ''.
        pct_samples (Union[None, Unset, float]):
        score (Union[None, Unset, float]):
        suggestion (Union[None, Unset, str]):
        xray_classname (Union[Unset, str]):  Default: ''.
    """

    alert_id: int
    dismissed: bool
    id: str
    message_fields: "XRayCardDBMessageFields"
    xray_class: XRayClass
    extra_filters: Union["ExtraAlertFilters", None, Unset] = UNSET
    importance: Union[AlertImportance, None, Unset] = UNSET
    logic: Union["FilterParams", None, Unset] = UNSET
    message: Union[Unset, str] = ""
    pct_samples: Union[None, Unset, float] = UNSET
    score: Union[None, Unset, float] = UNSET
    suggestion: Union[None, Unset, str] = UNSET
    xray_classname: Union[Unset, str] = ""
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.extra_alert_filters import ExtraAlertFilters
        from ..models.filter_params import FilterParams

        alert_id = self.alert_id

        dismissed = self.dismissed

        id = self.id

        message_fields = self.message_fields.to_dict()

        xray_class = self.xray_class.value

        extra_filters: Union[None, Unset, dict[str, Any]]
        if isinstance(self.extra_filters, Unset):
            extra_filters = UNSET
        elif isinstance(self.extra_filters, ExtraAlertFilters):
            extra_filters = self.extra_filters.to_dict()
        else:
            extra_filters = self.extra_filters

        importance: Union[None, Unset, str]
        if isinstance(self.importance, Unset):
            importance = UNSET
        elif isinstance(self.importance, AlertImportance):
            importance = self.importance.value
        else:
            importance = self.importance

        logic: Union[None, Unset, dict[str, Any]]
        if isinstance(self.logic, Unset):
            logic = UNSET
        elif isinstance(self.logic, FilterParams):
            logic = self.logic.to_dict()
        else:
            logic = self.logic

        message = self.message

        pct_samples: Union[None, Unset, float]
        if isinstance(self.pct_samples, Unset):
            pct_samples = UNSET
        else:
            pct_samples = self.pct_samples

        score: Union[None, Unset, float]
        if isinstance(self.score, Unset):
            score = UNSET
        else:
            score = self.score

        suggestion: Union[None, Unset, str]
        if isinstance(self.suggestion, Unset):
            suggestion = UNSET
        else:
            suggestion = self.suggestion

        xray_classname = self.xray_classname

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "alert_id": alert_id,
                "dismissed": dismissed,
                "id": id,
                "message_fields": message_fields,
                "xray_class": xray_class,
            }
        )
        if extra_filters is not UNSET:
            field_dict["extra_filters"] = extra_filters
        if importance is not UNSET:
            field_dict["importance"] = importance
        if logic is not UNSET:
            field_dict["logic"] = logic
        if message is not UNSET:
            field_dict["message"] = message
        if pct_samples is not UNSET:
            field_dict["pct_samples"] = pct_samples
        if score is not UNSET:
            field_dict["score"] = score
        if suggestion is not UNSET:
            field_dict["suggestion"] = suggestion
        if xray_classname is not UNSET:
            field_dict["xray_classname"] = xray_classname

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.extra_alert_filters import ExtraAlertFilters
        from ..models.filter_params import FilterParams
        from ..models.x_ray_card_db_message_fields import XRayCardDBMessageFields

        d = src_dict.copy()
        alert_id = d.pop("alert_id")

        dismissed = d.pop("dismissed")

        id = d.pop("id")

        message_fields = XRayCardDBMessageFields.from_dict(d.pop("message_fields"))

        xray_class = XRayClass(d.pop("xray_class"))

        def _parse_extra_filters(data: object) -> Union["ExtraAlertFilters", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                extra_filters_type_0 = ExtraAlertFilters.from_dict(data)

                return extra_filters_type_0
            except:  # noqa: E722
                pass
            return cast(Union["ExtraAlertFilters", None, Unset], data)

        extra_filters = _parse_extra_filters(d.pop("extra_filters", UNSET))

        def _parse_importance(data: object) -> Union[AlertImportance, None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                importance_type_0 = AlertImportance(data)

                return importance_type_0
            except:  # noqa: E722
                pass
            return cast(Union[AlertImportance, None, Unset], data)

        importance = _parse_importance(d.pop("importance", UNSET))

        def _parse_logic(data: object) -> Union["FilterParams", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                logic_type_0 = FilterParams.from_dict(data)

                return logic_type_0
            except:  # noqa: E722
                pass
            return cast(Union["FilterParams", None, Unset], data)

        logic = _parse_logic(d.pop("logic", UNSET))

        message = d.pop("message", UNSET)

        def _parse_pct_samples(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        pct_samples = _parse_pct_samples(d.pop("pct_samples", UNSET))

        def _parse_score(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        score = _parse_score(d.pop("score", UNSET))

        def _parse_suggestion(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        suggestion = _parse_suggestion(d.pop("suggestion", UNSET))

        xray_classname = d.pop("xray_classname", UNSET)

        x_ray_card_db = cls(
            alert_id=alert_id,
            dismissed=dismissed,
            id=id,
            message_fields=message_fields,
            xray_class=xray_class,
            extra_filters=extra_filters,
            importance=importance,
            logic=logic,
            message=message,
            pct_samples=pct_samples,
            score=score,
            suggestion=suggestion,
            xray_classname=xray_classname,
        )

        x_ray_card_db.additional_properties = d
        return x_ray_card_db

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.recompute_settings_observe_base import RecomputeSettingsObserveBase
    from ..models.recompute_settings_project import RecomputeSettingsProject
    from ..models.recompute_settings_runs import RecomputeSettingsRuns
    from ..models.scorers_config import ScorersConfig


T = TypeVar("T", bound="CreateComputeScoreRequest")


@_attrs_define
class CreateComputeScoreRequest:
    """
    Attributes:
        recompute_settings (Union['RecomputeSettingsObserveBase', 'RecomputeSettingsProject', 'RecomputeSettingsRuns']):
        scorers_config (ScorersConfig):
    """

    recompute_settings: Union["RecomputeSettingsObserveBase", "RecomputeSettingsProject", "RecomputeSettingsRuns"]
    scorers_config: "ScorersConfig"
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.recompute_settings_project import RecomputeSettingsProject
        from ..models.recompute_settings_runs import RecomputeSettingsRuns

        recompute_settings: dict[str, Any]
        if isinstance(self.recompute_settings, RecomputeSettingsRuns):
            recompute_settings = self.recompute_settings.to_dict()
        elif isinstance(self.recompute_settings, RecomputeSettingsProject):
            recompute_settings = self.recompute_settings.to_dict()
        else:
            recompute_settings = self.recompute_settings.to_dict()

        scorers_config = self.scorers_config.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"recompute_settings": recompute_settings, "scorers_config": scorers_config})

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.recompute_settings_observe_base import RecomputeSettingsObserveBase
        from ..models.recompute_settings_project import RecomputeSettingsProject
        from ..models.recompute_settings_runs import RecomputeSettingsRuns
        from ..models.scorers_config import ScorersConfig

        d = src_dict.copy()

        def _parse_recompute_settings(
            data: object,
        ) -> Union["RecomputeSettingsObserveBase", "RecomputeSettingsProject", "RecomputeSettingsRuns"]:
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                recompute_settings_type_0 = RecomputeSettingsRuns.from_dict(data)

                return recompute_settings_type_0
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                recompute_settings_type_1 = RecomputeSettingsProject.from_dict(data)

                return recompute_settings_type_1
            except:  # noqa: E722
                pass
            if not isinstance(data, dict):
                raise TypeError()
            recompute_settings_type_2 = RecomputeSettingsObserveBase.from_dict(data)

            return recompute_settings_type_2

        recompute_settings = _parse_recompute_settings(d.pop("recompute_settings"))

        scorers_config = ScorersConfig.from_dict(d.pop("scorers_config"))

        create_compute_score_request = cls(recompute_settings=recompute_settings, scorers_config=scorers_config)

        create_compute_score_request.additional_properties = d
        return create_compute_score_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.user_metadata_filter import UserMetadataFilter


T = TypeVar("T", bound="GetObserveWorkflowsRequest")


@_attrs_define
class GetObserveWorkflowsRequest:
    """
    Attributes:
        filters (Union[Unset, list['UserMetadataFilter']]):
        limit (Union[Unset, int]):  Default: 100.
        starting_token (Union[Unset, int]):  Default: 0.
    """

    filters: Union[Unset, list["UserMetadataFilter"]] = UNSET
    limit: Union[Unset, int] = 100
    starting_token: Union[Unset, int] = 0
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        filters: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.filters, Unset):
            filters = []
            for filters_item_data in self.filters:
                filters_item = filters_item_data.to_dict()
                filters.append(filters_item)

        limit = self.limit

        starting_token = self.starting_token

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if filters is not UNSET:
            field_dict["filters"] = filters
        if limit is not UNSET:
            field_dict["limit"] = limit
        if starting_token is not UNSET:
            field_dict["starting_token"] = starting_token

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.user_metadata_filter import UserMetadataFilter

        d = src_dict.copy()
        filters = []
        _filters = d.pop("filters", UNSET)
        for filters_item_data in _filters or []:
            filters_item = UserMetadataFilter.from_dict(filters_item_data)

            filters.append(filters_item)

        limit = d.pop("limit", UNSET)

        starting_token = d.pop("starting_token", UNSET)

        get_observe_workflows_request = cls(filters=filters, limit=limit, starting_token=starting_token)

        get_observe_workflows_request.additional_properties = d
        return get_observe_workflows_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="PutPromptScoreResponse")


@_attrs_define
class PutPromptScoreResponse:
    """
    Attributes:
        job_id (str):
        project_id (str):
        run_id (str):
        scorer_name (str):
    """

    job_id: str
    project_id: str
    run_id: str
    scorer_name: str
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        job_id = self.job_id

        project_id = self.project_id

        run_id = self.run_id

        scorer_name = self.scorer_name

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"job_id": job_id, "project_id": project_id, "run_id": run_id, "scorer_name": scorer_name})

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        job_id = d.pop("job_id")

        project_id = d.pop("project_id")

        run_id = d.pop("run_id")

        scorer_name = d.pop("scorer_name")

        put_prompt_score_response = cls(job_id=job_id, project_id=project_id, run_id=run_id, scorer_name=scorer_name)

        put_prompt_score_response.additional_properties = d
        return put_prompt_score_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.polygon_size import PolygonSize
from ..models.sem_seg_error_type import SemSegErrorType
from ..types import UNSET, Unset

T = TypeVar("T", bound="PolygonData")


@_attrs_define
class PolygonData:
    """
    Attributes:
        contours (list[Any]):
        data_error_potential (float):
        id (int):
        is_active (bool):
        is_gold (bool):
        is_pred (bool):
        accuracy (Union[None, Unset, float]):
        area (Union[None, Unset, int]):
        background_error_pct (Union[None, Unset, float]):
        error_type (Union[None, SemSegErrorType, Unset]):
        gold (Union[None, Unset, str]):
        pred (Union[None, Unset, str]):
        size (Union[None, PolygonSize, Unset]):
    """

    contours: list[Any]
    data_error_potential: float
    id: int
    is_active: bool
    is_gold: bool
    is_pred: bool
    accuracy: Union[None, Unset, float] = UNSET
    area: Union[None, Unset, int] = UNSET
    background_error_pct: Union[None, Unset, float] = UNSET
    error_type: Union[None, SemSegErrorType, Unset] = UNSET
    gold: Union[None, Unset, str] = UNSET
    pred: Union[None, Unset, str] = UNSET
    size: Union[None, PolygonSize, Unset] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        contours = self.contours

        data_error_potential = self.data_error_potential

        id = self.id

        is_active = self.is_active

        is_gold = self.is_gold

        is_pred = self.is_pred

        accuracy: Union[None, Unset, float]
        if isinstance(self.accuracy, Unset):
            accuracy = UNSET
        else:
            accuracy = self.accuracy

        area: Union[None, Unset, int]
        if isinstance(self.area, Unset):
            area = UNSET
        else:
            area = self.area

        background_error_pct: Union[None, Unset, float]
        if isinstance(self.background_error_pct, Unset):
            background_error_pct = UNSET
        else:
            background_error_pct = self.background_error_pct

        error_type: Union[None, Unset, str]
        if isinstance(self.error_type, Unset):
            error_type = UNSET
        elif isinstance(self.error_type, SemSegErrorType):
            error_type = self.error_type.value
        else:
            error_type = self.error_type

        gold: Union[None, Unset, str]
        if isinstance(self.gold, Unset):
            gold = UNSET
        else:
            gold = self.gold

        pred: Union[None, Unset, str]
        if isinstance(self.pred, Unset):
            pred = UNSET
        else:
            pred = self.pred

        size: Union[None, Unset, str]
        if isinstance(self.size, Unset):
            size = UNSET
        elif isinstance(self.size, PolygonSize):
            size = self.size.value
        else:
            size = self.size

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "contours": contours,
                "data_error_potential": data_error_potential,
                "id": id,
                "is_active": is_active,
                "is_gold": is_gold,
                "is_pred": is_pred,
            }
        )
        if accuracy is not UNSET:
            field_dict["accuracy"] = accuracy
        if area is not UNSET:
            field_dict["area"] = area
        if background_error_pct is not UNSET:
            field_dict["background_error_pct"] = background_error_pct
        if error_type is not UNSET:
            field_dict["error_type"] = error_type
        if gold is not UNSET:
            field_dict["gold"] = gold
        if pred is not UNSET:
            field_dict["pred"] = pred
        if size is not UNSET:
            field_dict["size"] = size

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        contours = cast(list[Any], d.pop("contours"))

        data_error_potential = d.pop("data_error_potential")

        id = d.pop("id")

        is_active = d.pop("is_active")

        is_gold = d.pop("is_gold")

        is_pred = d.pop("is_pred")

        def _parse_accuracy(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        accuracy = _parse_accuracy(d.pop("accuracy", UNSET))

        def _parse_area(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        area = _parse_area(d.pop("area", UNSET))

        def _parse_background_error_pct(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        background_error_pct = _parse_background_error_pct(d.pop("background_error_pct", UNSET))

        def _parse_error_type(data: object) -> Union[None, SemSegErrorType, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                error_type_type_0 = SemSegErrorType(data)

                return error_type_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, SemSegErrorType, Unset], data)

        error_type = _parse_error_type(d.pop("error_type", UNSET))

        def _parse_gold(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        gold = _parse_gold(d.pop("gold", UNSET))

        def _parse_pred(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        pred = _parse_pred(d.pop("pred", UNSET))

        def _parse_size(data: object) -> Union[None, PolygonSize, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                size_type_0 = PolygonSize(data)

                return size_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, PolygonSize, Unset], data)

        size = _parse_size(d.pop("size", UNSET))

        polygon_data = cls(
            contours=contours,
            data_error_potential=data_error_potential,
            id=id,
            is_active=is_active,
            is_gold=is_gold,
            is_pred=is_pred,
            accuracy=accuracy,
            area=area,
            background_error_pct=background_error_pct,
            error_type=error_type,
            gold=gold,
            pred=pred,
            size=size,
        )

        polygon_data.additional_properties = d
        return polygon_data

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="MetaInfo")


@_attrs_define
class MetaInfo:
    """Class for describing a metadata column.

    Metadata columns and tabular data columns are similar, but metadata columns

    a meta column is categorical if it is of string/object type with <= 50 unique values a meta column is continuous if
    it is of float/int type

    Thus, a column of string type with > 50 unique values is neither continuous nor categorical in our case.

        Attributes:
            is_categorical (bool):
            is_continuous (bool):
            name (str):
            high_cardinality (Union[None, Unset, bool]):
            max_ (Union[None, Unset, float]):
            mean (Union[None, Unset, float]):
            min_ (Union[None, Unset, float]):
            missing (Union[None, Unset, int]):
            missing_pct (Union[None, Unset, float]):
            negative (Union[None, Unset, int]):
            negative_pct (Union[None, Unset, float]):
            unique_count (Union[None, Unset, int]):
            unique_count_full (Union[None, Unset, int]):
            unique_values (Union[None, Unset, list[Any]]):
            unique_values_full (Union[None, Unset, list[Any]]):
            zeros (Union[None, Unset, int]):
            zeros_pct (Union[None, Unset, float]):
    """

    is_categorical: bool
    is_continuous: bool
    name: str
    high_cardinality: Union[None, Unset, bool] = UNSET
    max_: Union[None, Unset, float] = UNSET
    mean: Union[None, Unset, float] = UNSET
    min_: Union[None, Unset, float] = UNSET
    missing: Union[None, Unset, int] = UNSET
    missing_pct: Union[None, Unset, float] = UNSET
    negative: Union[None, Unset, int] = UNSET
    negative_pct: Union[None, Unset, float] = UNSET
    unique_count: Union[None, Unset, int] = UNSET
    unique_count_full: Union[None, Unset, int] = UNSET
    unique_values: Union[None, Unset, list[Any]] = UNSET
    unique_values_full: Union[None, Unset, list[Any]] = UNSET
    zeros: Union[None, Unset, int] = UNSET
    zeros_pct: Union[None, Unset, float] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        is_categorical = self.is_categorical

        is_continuous = self.is_continuous

        name = self.name

        high_cardinality: Union[None, Unset, bool]
        if isinstance(self.high_cardinality, Unset):
            high_cardinality = UNSET
        else:
            high_cardinality = self.high_cardinality

        max_: Union[None, Unset, float]
        if isinstance(self.max_, Unset):
            max_ = UNSET
        else:
            max_ = self.max_

        mean: Union[None, Unset, float]
        if isinstance(self.mean, Unset):
            mean = UNSET
        else:
            mean = self.mean

        min_: Union[None, Unset, float]
        if isinstance(self.min_, Unset):
            min_ = UNSET
        else:
            min_ = self.min_

        missing: Union[None, Unset, int]
        if isinstance(self.missing, Unset):
            missing = UNSET
        else:
            missing = self.missing

        missing_pct: Union[None, Unset, float]
        if isinstance(self.missing_pct, Unset):
            missing_pct = UNSET
        else:
            missing_pct = self.missing_pct

        negative: Union[None, Unset, int]
        if isinstance(self.negative, Unset):
            negative = UNSET
        else:
            negative = self.negative

        negative_pct: Union[None, Unset, float]
        if isinstance(self.negative_pct, Unset):
            negative_pct = UNSET
        else:
            negative_pct = self.negative_pct

        unique_count: Union[None, Unset, int]
        if isinstance(self.unique_count, Unset):
            unique_count = UNSET
        else:
            unique_count = self.unique_count

        unique_count_full: Union[None, Unset, int]
        if isinstance(self.unique_count_full, Unset):
            unique_count_full = UNSET
        else:
            unique_count_full = self.unique_count_full

        unique_values: Union[None, Unset, list[Any]]
        if isinstance(self.unique_values, Unset):
            unique_values = UNSET
        elif isinstance(self.unique_values, list):
            unique_values = self.unique_values

        else:
            unique_values = self.unique_values

        unique_values_full: Union[None, Unset, list[Any]]
        if isinstance(self.unique_values_full, Unset):
            unique_values_full = UNSET
        elif isinstance(self.unique_values_full, list):
            unique_values_full = self.unique_values_full

        else:
            unique_values_full = self.unique_values_full

        zeros: Union[None, Unset, int]
        if isinstance(self.zeros, Unset):
            zeros = UNSET
        else:
            zeros = self.zeros

        zeros_pct: Union[None, Unset, float]
        if isinstance(self.zeros_pct, Unset):
            zeros_pct = UNSET
        else:
            zeros_pct = self.zeros_pct

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"is_categorical": is_categorical, "is_continuous": is_continuous, "name": name})
        if high_cardinality is not UNSET:
            field_dict["high_cardinality"] = high_cardinality
        if max_ is not UNSET:
            field_dict["max"] = max_
        if mean is not UNSET:
            field_dict["mean"] = mean
        if min_ is not UNSET:
            field_dict["min"] = min_
        if missing is not UNSET:
            field_dict["missing"] = missing
        if missing_pct is not UNSET:
            field_dict["missing_pct"] = missing_pct
        if negative is not UNSET:
            field_dict["negative"] = negative
        if negative_pct is not UNSET:
            field_dict["negative_pct"] = negative_pct
        if unique_count is not UNSET:
            field_dict["unique_count"] = unique_count
        if unique_count_full is not UNSET:
            field_dict["unique_count_full"] = unique_count_full
        if unique_values is not UNSET:
            field_dict["unique_values"] = unique_values
        if unique_values_full is not UNSET:
            field_dict["unique_values_full"] = unique_values_full
        if zeros is not UNSET:
            field_dict["zeros"] = zeros
        if zeros_pct is not UNSET:
            field_dict["zeros_pct"] = zeros_pct

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        is_categorical = d.pop("is_categorical")

        is_continuous = d.pop("is_continuous")

        name = d.pop("name")

        def _parse_high_cardinality(data: object) -> Union[None, Unset, bool]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, bool], data)

        high_cardinality = _parse_high_cardinality(d.pop("high_cardinality", UNSET))

        def _parse_max_(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        max_ = _parse_max_(d.pop("max", UNSET))

        def _parse_mean(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        mean = _parse_mean(d.pop("mean", UNSET))

        def _parse_min_(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        min_ = _parse_min_(d.pop("min", UNSET))

        def _parse_missing(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        missing = _parse_missing(d.pop("missing", UNSET))

        def _parse_missing_pct(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        missing_pct = _parse_missing_pct(d.pop("missing_pct", UNSET))

        def _parse_negative(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        negative = _parse_negative(d.pop("negative", UNSET))

        def _parse_negative_pct(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        negative_pct = _parse_negative_pct(d.pop("negative_pct", UNSET))

        def _parse_unique_count(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        unique_count = _parse_unique_count(d.pop("unique_count", UNSET))

        def _parse_unique_count_full(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        unique_count_full = _parse_unique_count_full(d.pop("unique_count_full", UNSET))

        def _parse_unique_values(data: object) -> Union[None, Unset, list[Any]]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                unique_values_type_0 = cast(list[Any], data)

                return unique_values_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, list[Any]], data)

        unique_values = _parse_unique_values(d.pop("unique_values", UNSET))

        def _parse_unique_values_full(data: object) -> Union[None, Unset, list[Any]]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                unique_values_full_type_0 = cast(list[Any], data)

                return unique_values_full_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, list[Any]], data)

        unique_values_full = _parse_unique_values_full(d.pop("unique_values_full", UNSET))

        def _parse_zeros(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        zeros = _parse_zeros(d.pop("zeros", UNSET))

        def _parse_zeros_pct(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        zeros_pct = _parse_zeros_pct(d.pop("zeros_pct", UNSET))

        meta_info = cls(
            is_categorical=is_categorical,
            is_continuous=is_continuous,
            name=name,
            high_cardinality=high_cardinality,
            max_=max_,
            mean=mean,
            min_=min_,
            missing=missing,
            missing_pct=missing_pct,
            negative=negative,
            negative_pct=negative_pct,
            unique_count=unique_count,
            unique_count_full=unique_count_full,
            unique_values=unique_values,
            unique_values_full=unique_values_full,
            zeros=zeros,
            zeros_pct=zeros_pct,
        )

        meta_info.additional_properties = d
        return meta_info

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.cartograph_cluster import CartographCluster


T = TypeVar("T", bound="CartographClusterResponse")


@_attrs_define
class CartographClusterResponse:
    """
    Attributes:
        cartograph_clusters (list['CartographCluster']):
    """

    cartograph_clusters: list["CartographCluster"]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        cartograph_clusters = []
        for cartograph_clusters_item_data in self.cartograph_clusters:
            cartograph_clusters_item = cartograph_clusters_item_data.to_dict()
            cartograph_clusters.append(cartograph_clusters_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"cartograph_clusters": cartograph_clusters})

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.cartograph_cluster import CartographCluster

        d = src_dict.copy()
        cartograph_clusters = []
        _cartograph_clusters = d.pop("cartograph_clusters")
        for cartograph_clusters_item_data in _cartograph_clusters:
            cartograph_clusters_item = CartographCluster.from_dict(cartograph_clusters_item_data)

            cartograph_clusters.append(cartograph_clusters_item)

        cartograph_cluster_response = cls(cartograph_clusters=cartograph_clusters)

        cartograph_cluster_response.additional_properties = d
        return cartograph_cluster_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import asyncio
import functools
import logging
from logging import Logger
from typing import Any, Callable


def warn_catch_exception(logger: Logger = logging.getLogger(__name__), exception: type[Exception] = Exception) -> Any:
    """
    A function wrapper that catches exceptions and logs them to the logger.
    Args:
        exception:
        logger:

    Returns:

    """

    def wrapper(f: Callable) -> Callable:
        @functools.wraps(f)
        def inner(*args: Any, **kwargs: Any) -> Any:
            try:
                result = f(*args, **kwargs)
            except exception as err:
                logger.warning(f"Error occurred during execution: {f.__name__}: {err}")
            else:
                return result

        return inner

    return wrapper


def async_warn_catch_exception(
    logger: Logger = logging.getLogger(__name__), exception: type[Exception] = Exception
) -> Any:
    """
    A function wrapper that catches exceptions and logs them to the logger.
    Args:
        exception:
        logger:

    Returns:

    """

    def wrapper(f: Callable) -> Callable:
        @functools.wraps(f)
        async def inner(*args: Any, **kwargs: Any) -> Any:
            try:
                result = await f(*args, **kwargs)
            except exception as err:
                logger.warning(f"Error occurred during execution: {f.__name__}: {err}")
            else:
                return result

        return inner

    return wrapper


class DecorateAllMethods:
    def __init_subclass__(cls, **kwargs: Any) -> None:
        super().__init_subclass__(**kwargs)
        for attr, f in cls.__dict__.items():
            if attr == "__init__":
                continue
            if callable(f):
                if asyncio.iscoroutinefunction(f):
                    setattr(cls, attr, async_warn_catch_exception()(f))
                else:
                    setattr(cls, attr, warn_catch_exception()(f))



from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.insight_type import InsightType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.metric_insight_categories import MetricInsightCategories
    from ..models.metric_threshold import MetricThreshold


T = TypeVar("T", bound="MetricInsight")


@_attrs_define
class MetricInsight:
    """
    Attributes:
        categories (MetricInsightCategories):
        column_name (str):
        insight_type (InsightType):
        title (str):
        aggregate (Union[None, Unset, float]):
        metric_threshold (Union['MetricThreshold', None, Unset]):
    """

    categories: "MetricInsightCategories"
    column_name: str
    insight_type: InsightType
    title: str
    aggregate: Union[None, Unset, float] = UNSET
    metric_threshold: Union["MetricThreshold", None, Unset] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.metric_threshold import MetricThreshold

        categories = self.categories.to_dict()

        column_name = self.column_name

        insight_type = self.insight_type.value

        title = self.title

        aggregate: Union[None, Unset, float]
        if isinstance(self.aggregate, Unset):
            aggregate = UNSET
        else:
            aggregate = self.aggregate

        metric_threshold: Union[None, Unset, dict[str, Any]]
        if isinstance(self.metric_threshold, Unset):
            metric_threshold = UNSET
        elif isinstance(self.metric_threshold, MetricThreshold):
            metric_threshold = self.metric_threshold.to_dict()
        else:
            metric_threshold = self.metric_threshold

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {"categories": categories, "column_name": column_name, "insight_type": insight_type, "title": title}
        )
        if aggregate is not UNSET:
            field_dict["aggregate"] = aggregate
        if metric_threshold is not UNSET:
            field_dict["metric_threshold"] = metric_threshold

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.metric_insight_categories import MetricInsightCategories
        from ..models.metric_threshold import MetricThreshold

        d = src_dict.copy()
        categories = MetricInsightCategories.from_dict(d.pop("categories"))

        column_name = d.pop("column_name")

        insight_type = InsightType(d.pop("insight_type"))

        title = d.pop("title")

        def _parse_aggregate(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        aggregate = _parse_aggregate(d.pop("aggregate", UNSET))

        def _parse_metric_threshold(data: object) -> Union["MetricThreshold", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                metric_threshold_type_0 = MetricThreshold.from_dict(data)

                return metric_threshold_type_0
            except:  # noqa: E722
                pass
            return cast(Union["MetricThreshold", None, Unset], data)

        metric_threshold = _parse_metric_threshold(d.pop("metric_threshold", UNSET))

        metric_insight = cls(
            categories=categories,
            column_name=column_name,
            insight_type=insight_type,
            title=title,
            aggregate=aggregate,
            metric_threshold=metric_threshold,
        )

        metric_insight.additional_properties = d
        return metric_insight

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.create_metric_critique_request import CreateMetricCritiqueRequest
    from ..models.recompute_settings_observe import RecomputeSettingsObserve
    from ..models.recompute_settings_project import RecomputeSettingsProject
    from ..models.recompute_settings_runs import RecomputeSettingsRuns


T = TypeVar("T", bound="CreateMetricCritiquesRequest")


@_attrs_define
class CreateMetricCritiquesRequest:
    """
    Attributes:
        critiques (list['CreateMetricCritiqueRequest']):
        metric (str):
        recompute_settings (Union['RecomputeSettingsObserve', 'RecomputeSettingsProject', 'RecomputeSettingsRuns', None,
            Unset]):
    """

    critiques: list["CreateMetricCritiqueRequest"]
    metric: str
    recompute_settings: Union[
        "RecomputeSettingsObserve", "RecomputeSettingsProject", "RecomputeSettingsRuns", None, Unset
    ] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.recompute_settings_observe import RecomputeSettingsObserve
        from ..models.recompute_settings_project import RecomputeSettingsProject
        from ..models.recompute_settings_runs import RecomputeSettingsRuns

        critiques = []
        for critiques_item_data in self.critiques:
            critiques_item = critiques_item_data.to_dict()
            critiques.append(critiques_item)

        metric = self.metric

        recompute_settings: Union[None, Unset, dict[str, Any]]
        if isinstance(self.recompute_settings, Unset):
            recompute_settings = UNSET
        elif isinstance(self.recompute_settings, RecomputeSettingsRuns):
            recompute_settings = self.recompute_settings.to_dict()
        elif isinstance(self.recompute_settings, RecomputeSettingsProject):
            recompute_settings = self.recompute_settings.to_dict()
        elif isinstance(self.recompute_settings, RecomputeSettingsObserve):
            recompute_settings = self.recompute_settings.to_dict()
        else:
            recompute_settings = self.recompute_settings

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"critiques": critiques, "metric": metric})
        if recompute_settings is not UNSET:
            field_dict["recompute_settings"] = recompute_settings

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.create_metric_critique_request import CreateMetricCritiqueRequest
        from ..models.recompute_settings_observe import RecomputeSettingsObserve
        from ..models.recompute_settings_project import RecomputeSettingsProject
        from ..models.recompute_settings_runs import RecomputeSettingsRuns

        d = src_dict.copy()
        critiques = []
        _critiques = d.pop("critiques")
        for critiques_item_data in _critiques:
            critiques_item = CreateMetricCritiqueRequest.from_dict(critiques_item_data)

            critiques.append(critiques_item)

        metric = d.pop("metric")

        def _parse_recompute_settings(
            data: object,
        ) -> Union["RecomputeSettingsObserve", "RecomputeSettingsProject", "RecomputeSettingsRuns", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                recompute_settings_type_0_type_0 = RecomputeSettingsRuns.from_dict(data)

                return recompute_settings_type_0_type_0
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                recompute_settings_type_0_type_1 = RecomputeSettingsProject.from_dict(data)

                return recompute_settings_type_0_type_1
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                recompute_settings_type_0_type_2 = RecomputeSettingsObserve.from_dict(data)

                return recompute_settings_type_0_type_2
            except:  # noqa: E722
                pass
            return cast(
                Union["RecomputeSettingsObserve", "RecomputeSettingsProject", "RecomputeSettingsRuns", None, Unset],
                data,
            )

        recompute_settings = _parse_recompute_settings(d.pop("recompute_settings", UNSET))

        create_metric_critiques_request = cls(critiques=critiques, metric=metric, recompute_settings=recompute_settings)

        create_metric_critiques_request.additional_properties = d
        return create_metric_critiques_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="DatabricksIntegrationCreate")


@_attrs_define
class DatabricksIntegrationCreate:
    """
    Attributes:
        hostname (str):
        token (str):
        default_catalog_name (Union[None, Unset, str]):
        llm (Union[Unset, bool]):  Default: False.
        path (Union[None, Unset, str]):
        storage (Union[Unset, bool]):  Default: False.
    """

    hostname: str
    token: str
    default_catalog_name: Union[None, Unset, str] = UNSET
    llm: Union[Unset, bool] = False
    path: Union[None, Unset, str] = UNSET
    storage: Union[Unset, bool] = False
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        hostname = self.hostname

        token = self.token

        default_catalog_name: Union[None, Unset, str]
        if isinstance(self.default_catalog_name, Unset):
            default_catalog_name = UNSET
        else:
            default_catalog_name = self.default_catalog_name

        llm = self.llm

        path: Union[None, Unset, str]
        if isinstance(self.path, Unset):
            path = UNSET
        else:
            path = self.path

        storage = self.storage

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"hostname": hostname, "token": token})
        if default_catalog_name is not UNSET:
            field_dict["default_catalog_name"] = default_catalog_name
        if llm is not UNSET:
            field_dict["llm"] = llm
        if path is not UNSET:
            field_dict["path"] = path
        if storage is not UNSET:
            field_dict["storage"] = storage

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        hostname = d.pop("hostname")

        token = d.pop("token")

        def _parse_default_catalog_name(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        default_catalog_name = _parse_default_catalog_name(d.pop("default_catalog_name", UNSET))

        llm = d.pop("llm", UNSET)

        def _parse_path(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        path = _parse_path(d.pop("path", UNSET))

        storage = d.pop("storage", UNSET)

        databricks_integration_create = cls(
            hostname=hostname,
            token=token,
            default_catalog_name=default_catalog_name,
            llm=llm,
            path=path,
            storage=storage,
        )

        databricks_integration_create.additional_properties = d
        return databricks_integration_create

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.aggregator import Aggregator
from ..models.alert_condition_type import AlertConditionType
from ..models.operator import Operator
from ..types import UNSET, Unset

T = TypeVar("T", bound="AlertCondition")


@_attrs_define
class AlertCondition:
    """
    Attributes:
        aggregation (Union[Aggregator, str]):
        field (str):
        operator (Operator):
        value (Union[float, int, str]):
        window (int):
        condition_type (Union[Unset, AlertConditionType]):
        filter_operator (Union[None, Operator, Unset]):
        filter_value (Union[None, Unset, float, int, str]):
    """

    aggregation: Union[Aggregator, str]
    field: str
    operator: Operator
    value: Union[float, int, str]
    window: int
    condition_type: Union[Unset, AlertConditionType] = UNSET
    filter_operator: Union[None, Operator, Unset] = UNSET
    filter_value: Union[None, Unset, float, int, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        aggregation: str
        if isinstance(self.aggregation, Aggregator):
            aggregation = self.aggregation.value
        else:
            aggregation = self.aggregation

        field = self.field

        operator = self.operator.value

        value: Union[float, int, str]
        value = self.value

        window = self.window

        condition_type: Union[Unset, str] = UNSET
        if not isinstance(self.condition_type, Unset):
            condition_type = self.condition_type.value

        filter_operator: Union[None, Unset, str]
        if isinstance(self.filter_operator, Unset):
            filter_operator = UNSET
        elif isinstance(self.filter_operator, Operator):
            filter_operator = self.filter_operator.value
        else:
            filter_operator = self.filter_operator

        filter_value: Union[None, Unset, float, int, str]
        if isinstance(self.filter_value, Unset):
            filter_value = UNSET
        else:
            filter_value = self.filter_value

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {"aggregation": aggregation, "field": field, "operator": operator, "value": value, "window": window}
        )
        if condition_type is not UNSET:
            field_dict["condition_type"] = condition_type
        if filter_operator is not UNSET:
            field_dict["filter_operator"] = filter_operator
        if filter_value is not UNSET:
            field_dict["filter_value"] = filter_value

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()

        def _parse_aggregation(data: object) -> Union[Aggregator, str]:
            try:
                if not isinstance(data, str):
                    raise TypeError()
                aggregation_type_0 = Aggregator(data)

                return aggregation_type_0
            except:  # noqa: E722
                pass
            return cast(Union[Aggregator, str], data)

        aggregation = _parse_aggregation(d.pop("aggregation"))

        field = d.pop("field")

        operator = Operator(d.pop("operator"))

        def _parse_value(data: object) -> Union[float, int, str]:
            return cast(Union[float, int, str], data)

        value = _parse_value(d.pop("value"))

        window = d.pop("window")

        _condition_type = d.pop("condition_type", UNSET)
        condition_type: Union[Unset, AlertConditionType]
        if isinstance(_condition_type, Unset):
            condition_type = UNSET
        else:
            condition_type = AlertConditionType(_condition_type)

        def _parse_filter_operator(data: object) -> Union[None, Operator, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                filter_operator_type_0 = Operator(data)

                return filter_operator_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Operator, Unset], data)

        filter_operator = _parse_filter_operator(d.pop("filter_operator", UNSET))

        def _parse_filter_value(data: object) -> Union[None, Unset, float, int, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float, int, str], data)

        filter_value = _parse_filter_value(d.pop("filter_value", UNSET))

        alert_condition = cls(
            aggregation=aggregation,
            field=field,
            operator=operator,
            value=value,
            window=window,
            condition_type=condition_type,
            filter_operator=filter_operator,
            filter_value=filter_value,
        )

        alert_condition.additional_properties = d
        return alert_condition

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="EvaluateRunResponse")


@_attrs_define
class EvaluateRunResponse:
    """
    Attributes:
        message (str):
        project_id (str):
        project_name (str):
        records_count (int):
        run_id (str):
        run_name (str):
        workflows_count (int):
    """

    message: str
    project_id: str
    project_name: str
    records_count: int
    run_id: str
    run_name: str
    workflows_count: int
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        message = self.message

        project_id = self.project_id

        project_name = self.project_name

        records_count = self.records_count

        run_id = self.run_id

        run_name = self.run_name

        workflows_count = self.workflows_count

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "message": message,
                "project_id": project_id,
                "project_name": project_name,
                "records_count": records_count,
                "run_id": run_id,
                "run_name": run_name,
                "workflows_count": workflows_count,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        message = d.pop("message")

        project_id = d.pop("project_id")

        project_name = d.pop("project_name")

        records_count = d.pop("records_count")

        run_id = d.pop("run_id")

        run_name = d.pop("run_name")

        workflows_count = d.pop("workflows_count")

        evaluate_run_response = cls(
            message=message,
            project_id=project_id,
            project_name=project_name,
            records_count=records_count,
            run_id=run_id,
            run_name=run_name,
            workflows_count=workflows_count,
        )

        evaluate_run_response.additional_properties = d
        return evaluate_run_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from http import HTTPStatus
from typing import Any, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.http_validation_error import HTTPValidationError
from ...models.transaction_rows_request_body import TransactionRowsRequestBody
from ...types import UNSET, Response, Unset


def _get_kwargs(
    project_id: str,
    *,
    body: TransactionRowsRequestBody,
    start_time: Union[None, Unset, datetime.datetime] = UNSET,
    end_time: Union[None, Unset, datetime.datetime] = UNSET,
    chain_id: Union[None, Unset, str] = UNSET,
    limit: Union[Unset, int] = 50,
    offset: Union[Unset, int] = 0,
    include_chains: Union[Unset, bool] = False,
    return_col_schema: Union[Unset, bool] = True,
) -> dict[str, Any]:
    headers: dict[str, Any] = {}

    params: dict[str, Any] = {}

    json_start_time: Union[None, Unset, str]
    if isinstance(start_time, Unset):
        json_start_time = UNSET
    elif isinstance(start_time, datetime.datetime):
        json_start_time = start_time.isoformat()
    else:
        json_start_time = start_time
    params["start_time"] = json_start_time

    json_end_time: Union[None, Unset, str]
    if isinstance(end_time, Unset):
        json_end_time = UNSET
    elif isinstance(end_time, datetime.datetime):
        json_end_time = end_time.isoformat()
    else:
        json_end_time = end_time
    params["end_time"] = json_end_time

    json_chain_id: Union[None, Unset, str]
    if isinstance(chain_id, Unset):
        json_chain_id = UNSET
    else:
        json_chain_id = chain_id
    params["chain_id"] = json_chain_id

    params["limit"] = limit

    params["offset"] = offset

    params["include_chains"] = include_chains

    params["return_col_schema"] = return_col_schema

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {"method": "post", "url": f"/projects/{project_id}/observe/rows", "params": params}

    _body = body.to_dict()

    _kwargs["json"] = _body
    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[HTTPValidationError]:
    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[HTTPValidationError]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    project_id: str,
    *,
    client: AuthenticatedClient,
    body: TransactionRowsRequestBody,
    start_time: Union[None, Unset, datetime.datetime] = UNSET,
    end_time: Union[None, Unset, datetime.datetime] = UNSET,
    chain_id: Union[None, Unset, str] = UNSET,
    limit: Union[Unset, int] = 50,
    offset: Union[Unset, int] = 0,
    include_chains: Union[Unset, bool] = False,
    return_col_schema: Union[Unset, bool] = True,
) -> Response[HTTPValidationError]:
    """Get Rows

     Get all rows from monitor_records.

    Returns them in order of creation from newest to oldest.

    Args:
        project_id (str):
        start_time (Union[None, Unset, datetime.datetime]):
        end_time (Union[None, Unset, datetime.datetime]):
        chain_id (Union[None, Unset, str]):
        limit (Union[Unset, int]):  Default: 50.
        offset (Union[Unset, int]):  Default: 0.
        include_chains (Union[Unset, bool]):  Default: False.
        return_col_schema (Union[Unset, bool]):  Default: True.
        body (TransactionRowsRequestBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[HTTPValidationError]
    """

    kwargs = _get_kwargs(
        project_id=project_id,
        body=body,
        start_time=start_time,
        end_time=end_time,
        chain_id=chain_id,
        limit=limit,
        offset=offset,
        include_chains=include_chains,
        return_col_schema=return_col_schema,
    )

    response = client.get_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


def sync(
    project_id: str,
    *,
    client: AuthenticatedClient,
    body: TransactionRowsRequestBody,
    start_time: Union[None, Unset, datetime.datetime] = UNSET,
    end_time: Union[None, Unset, datetime.datetime] = UNSET,
    chain_id: Union[None, Unset, str] = UNSET,
    limit: Union[Unset, int] = 50,
    offset: Union[Unset, int] = 0,
    include_chains: Union[Unset, bool] = False,
    return_col_schema: Union[Unset, bool] = True,
) -> Optional[HTTPValidationError]:
    """Get Rows

     Get all rows from monitor_records.

    Returns them in order of creation from newest to oldest.

    Args:
        project_id (str):
        start_time (Union[None, Unset, datetime.datetime]):
        end_time (Union[None, Unset, datetime.datetime]):
        chain_id (Union[None, Unset, str]):
        limit (Union[Unset, int]):  Default: 50.
        offset (Union[Unset, int]):  Default: 0.
        include_chains (Union[Unset, bool]):  Default: False.
        return_col_schema (Union[Unset, bool]):  Default: True.
        body (TransactionRowsRequestBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        HTTPValidationError
    """

    return sync_detailed(
        project_id=project_id,
        client=client,
        body=body,
        start_time=start_time,
        end_time=end_time,
        chain_id=chain_id,
        limit=limit,
        offset=offset,
        include_chains=include_chains,
        return_col_schema=return_col_schema,
    ).parsed


async def asyncio_detailed(
    project_id: str,
    *,
    client: AuthenticatedClient,
    body: TransactionRowsRequestBody,
    start_time: Union[None, Unset, datetime.datetime] = UNSET,
    end_time: Union[None, Unset, datetime.datetime] = UNSET,
    chain_id: Union[None, Unset, str] = UNSET,
    limit: Union[Unset, int] = 50,
    offset: Union[Unset, int] = 0,
    include_chains: Union[Unset, bool] = False,
    return_col_schema: Union[Unset, bool] = True,
) -> Response[HTTPValidationError]:
    """Get Rows

     Get all rows from monitor_records.

    Returns them in order of creation from newest to oldest.

    Args:
        project_id (str):
        start_time (Union[None, Unset, datetime.datetime]):
        end_time (Union[None, Unset, datetime.datetime]):
        chain_id (Union[None, Unset, str]):
        limit (Union[Unset, int]):  Default: 50.
        offset (Union[Unset, int]):  Default: 0.
        include_chains (Union[Unset, bool]):  Default: False.
        return_col_schema (Union[Unset, bool]):  Default: True.
        body (TransactionRowsRequestBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[HTTPValidationError]
    """

    kwargs = _get_kwargs(
        project_id=project_id,
        body=body,
        start_time=start_time,
        end_time=end_time,
        chain_id=chain_id,
        limit=limit,
        offset=offset,
        include_chains=include_chains,
        return_col_schema=return_col_schema,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    project_id: str,
    *,
    client: AuthenticatedClient,
    body: TransactionRowsRequestBody,
    start_time: Union[None, Unset, datetime.datetime] = UNSET,
    end_time: Union[None, Unset, datetime.datetime] = UNSET,
    chain_id: Union[None, Unset, str] = UNSET,
    limit: Union[Unset, int] = 50,
    offset: Union[Unset, int] = 0,
    include_chains: Union[Unset, bool] = False,
    return_col_schema: Union[Unset, bool] = True,
) -> Optional[HTTPValidationError]:
    """Get Rows

     Get all rows from monitor_records.

    Returns them in order of creation from newest to oldest.

    Args:
        project_id (str):
        start_time (Union[None, Unset, datetime.datetime]):
        end_time (Union[None, Unset, datetime.datetime]):
        chain_id (Union[None, Unset, str]):
        limit (Union[Unset, int]):  Default: 50.
        offset (Union[Unset, int]):  Default: 0.
        include_chains (Union[Unset, bool]):  Default: False.
        return_col_schema (Union[Unset, bool]):  Default: True.
        body (TransactionRowsRequestBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        HTTPValidationError
    """

    return (
        await asyncio_detailed(
            project_id=project_id,
            client=client,
            body=body,
            start_time=start_time,
            end_time=end_time,
            chain_id=chain_id,
            limit=limit,
            offset=offset,
            include_chains=include_chains,
            return_col_schema=return_col_schema,
        )
    ).parsed
